from http import HTTPStatus
from typing import Any

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.error_response import ErrorResponse
from ...models.get_autocomplete_postal_code_mode import GetAutocompletePostalCodeMode
from ...models.get_autocomplete_show_item import GetAutocompleteShowItem
from ...models.get_autocomplete_types_item import GetAutocompleteTypesItem
from ...models.open_search_autocomplete_response import OpenSearchAutocompleteResponse
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    q: str,
    at: str | Unset = UNSET,
    in_: str | Unset = UNSET,
    postal_code_mode: GetAutocompletePostalCodeMode | Unset = UNSET,
    types: list[GetAutocompleteTypesItem] | Unset = UNSET,
    lang: list[str] | Unset = UNSET,
    limit: int | Unset = 5,
    political_view: str | Unset = UNSET,
    show: list[GetAutocompleteShowItem] | Unset = UNSET,
    x_request_id: str | Unset = UNSET,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}
    if not isinstance(x_request_id, Unset):
        headers["X-Request-ID"] = x_request_id

    params: dict[str, Any] = {}

    params["q"] = q

    params["at"] = at

    params["in"] = in_

    json_postal_code_mode: str | Unset = UNSET
    if not isinstance(postal_code_mode, Unset):
        json_postal_code_mode = postal_code_mode.value

    params["postalCodeMode"] = json_postal_code_mode

    json_types: list[str] | Unset = UNSET
    if not isinstance(types, Unset):
        json_types = []
        for types_item_data in types:
            types_item = types_item_data.value
            json_types.append(types_item)

    params["types"] = json_types

    json_lang: list[str] | Unset = UNSET
    if not isinstance(lang, Unset):
        json_lang = lang

    params["lang"] = json_lang

    params["limit"] = limit

    params["politicalView"] = political_view

    json_show: list[str] | Unset = UNSET
    if not isinstance(show, Unset):
        json_show = []
        for show_item_data in show:
            show_item = show_item_data.value
            json_show.append(show_item)

    params["show"] = json_show

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/autocomplete",
        "params": params,
    }

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> ErrorResponse | OpenSearchAutocompleteResponse | None:
    if response.status_code == 200:
        response_200 = OpenSearchAutocompleteResponse.from_dict(response.json())

        return response_200

    if response.status_code == 400:
        response_400 = ErrorResponse.from_dict(response.json())

        return response_400

    if response.status_code == 405:
        response_405 = ErrorResponse.from_dict(response.json())

        return response_405

    if response.status_code == 429:
        response_429 = ErrorResponse.from_dict(response.json())

        return response_429

    if response.status_code == 503:
        response_503 = ErrorResponse.from_dict(response.json())

        return response_503

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[ErrorResponse | OpenSearchAutocompleteResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient | Client,
    q: str,
    at: str | Unset = UNSET,
    in_: str | Unset = UNSET,
    postal_code_mode: GetAutocompletePostalCodeMode | Unset = UNSET,
    types: list[GetAutocompleteTypesItem] | Unset = UNSET,
    lang: list[str] | Unset = UNSET,
    limit: int | Unset = 5,
    political_view: str | Unset = UNSET,
    show: list[GetAutocompleteShowItem] | Unset = UNSET,
    x_request_id: str | Unset = UNSET,
) -> Response[ErrorResponse | OpenSearchAutocompleteResponse]:
    """Autocomplete

     This endpoint completes entered keystrokes to a valid street address or
    administrative area to speed-up entering address queries.

    Args:
        q (str):  Example: Berlin Pariser 20.
        at (str | Unset):
        in_ (str | Unset):
        postal_code_mode (GetAutocompletePostalCodeMode | Unset):
        types (list[GetAutocompleteTypesItem] | Unset):
        lang (list[str] | Unset):
        limit (int | Unset):  Default: 5.
        political_view (str | Unset):
        show (list[GetAutocompleteShowItem] | Unset):
        x_request_id (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ErrorResponse | OpenSearchAutocompleteResponse]
    """

    kwargs = _get_kwargs(
        q=q,
        at=at,
        in_=in_,
        postal_code_mode=postal_code_mode,
        types=types,
        lang=lang,
        limit=limit,
        political_view=political_view,
        show=show,
        x_request_id=x_request_id,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient | Client,
    q: str,
    at: str | Unset = UNSET,
    in_: str | Unset = UNSET,
    postal_code_mode: GetAutocompletePostalCodeMode | Unset = UNSET,
    types: list[GetAutocompleteTypesItem] | Unset = UNSET,
    lang: list[str] | Unset = UNSET,
    limit: int | Unset = 5,
    political_view: str | Unset = UNSET,
    show: list[GetAutocompleteShowItem] | Unset = UNSET,
    x_request_id: str | Unset = UNSET,
) -> ErrorResponse | OpenSearchAutocompleteResponse | None:
    """Autocomplete

     This endpoint completes entered keystrokes to a valid street address or
    administrative area to speed-up entering address queries.

    Args:
        q (str):  Example: Berlin Pariser 20.
        at (str | Unset):
        in_ (str | Unset):
        postal_code_mode (GetAutocompletePostalCodeMode | Unset):
        types (list[GetAutocompleteTypesItem] | Unset):
        lang (list[str] | Unset):
        limit (int | Unset):  Default: 5.
        political_view (str | Unset):
        show (list[GetAutocompleteShowItem] | Unset):
        x_request_id (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ErrorResponse | OpenSearchAutocompleteResponse
    """

    return sync_detailed(
        client=client,
        q=q,
        at=at,
        in_=in_,
        postal_code_mode=postal_code_mode,
        types=types,
        lang=lang,
        limit=limit,
        political_view=political_view,
        show=show,
        x_request_id=x_request_id,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient | Client,
    q: str,
    at: str | Unset = UNSET,
    in_: str | Unset = UNSET,
    postal_code_mode: GetAutocompletePostalCodeMode | Unset = UNSET,
    types: list[GetAutocompleteTypesItem] | Unset = UNSET,
    lang: list[str] | Unset = UNSET,
    limit: int | Unset = 5,
    political_view: str | Unset = UNSET,
    show: list[GetAutocompleteShowItem] | Unset = UNSET,
    x_request_id: str | Unset = UNSET,
) -> Response[ErrorResponse | OpenSearchAutocompleteResponse]:
    """Autocomplete

     This endpoint completes entered keystrokes to a valid street address or
    administrative area to speed-up entering address queries.

    Args:
        q (str):  Example: Berlin Pariser 20.
        at (str | Unset):
        in_ (str | Unset):
        postal_code_mode (GetAutocompletePostalCodeMode | Unset):
        types (list[GetAutocompleteTypesItem] | Unset):
        lang (list[str] | Unset):
        limit (int | Unset):  Default: 5.
        political_view (str | Unset):
        show (list[GetAutocompleteShowItem] | Unset):
        x_request_id (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ErrorResponse | OpenSearchAutocompleteResponse]
    """

    kwargs = _get_kwargs(
        q=q,
        at=at,
        in_=in_,
        postal_code_mode=postal_code_mode,
        types=types,
        lang=lang,
        limit=limit,
        political_view=political_view,
        show=show,
        x_request_id=x_request_id,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient | Client,
    q: str,
    at: str | Unset = UNSET,
    in_: str | Unset = UNSET,
    postal_code_mode: GetAutocompletePostalCodeMode | Unset = UNSET,
    types: list[GetAutocompleteTypesItem] | Unset = UNSET,
    lang: list[str] | Unset = UNSET,
    limit: int | Unset = 5,
    political_view: str | Unset = UNSET,
    show: list[GetAutocompleteShowItem] | Unset = UNSET,
    x_request_id: str | Unset = UNSET,
) -> ErrorResponse | OpenSearchAutocompleteResponse | None:
    """Autocomplete

     This endpoint completes entered keystrokes to a valid street address or
    administrative area to speed-up entering address queries.

    Args:
        q (str):  Example: Berlin Pariser 20.
        at (str | Unset):
        in_ (str | Unset):
        postal_code_mode (GetAutocompletePostalCodeMode | Unset):
        types (list[GetAutocompleteTypesItem] | Unset):
        lang (list[str] | Unset):
        limit (int | Unset):  Default: 5.
        political_view (str | Unset):
        show (list[GetAutocompleteShowItem] | Unset):
        x_request_id (str | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ErrorResponse | OpenSearchAutocompleteResponse
    """

    return (
        await asyncio_detailed(
            client=client,
            q=q,
            at=at,
            in_=in_,
            postal_code_mode=postal_code_mode,
            types=types,
            lang=lang,
            limit=limit,
            political_view=political_view,
            show=show,
            x_request_id=x_request_id,
        )
    ).parsed



import datetime
from http import HTTPStatus
from typing import Any

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.allow import Allow
from ...models.auth_error_response_schema import AuthErrorResponseSchema
from ...models.avoid import Avoid
from ...models.base_ev_empirical_model import BaseEVEmpiricalModel
from ...models.base_ev_physical_model import BaseEVPhysicalModel
from ...models.calculate_isolines_post_optimize_for import (
    CalculateIsolinesPostOptimizeFor,
)
from ...models.calculate_isolines_post_parameters import CalculateIsolinesPostParameters
from ...models.exclude import Exclude
from ...models.fuel import Fuel
from ...models.isoline_error_response import IsolineErrorResponse
from ...models.isoline_response import IsolineResponse
from ...models.not_allowed import NotAllowed
from ...models.range_ import Range
from ...models.router_mode import RouterMode
from ...models.routing_mode import RoutingMode
from ...models.server_internal import ServerInternal
from ...models.shape import Shape
from ...models.taxi import Taxi
from ...models.traffic import Traffic
from ...models.truck import Truck
from ...models.vehicle import Vehicle
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    body: CalculateIsolinesPostParameters | Unset = UNSET,
    range_: Range,
    transport_mode: RouterMode,
    origin: str | Unset = UNSET,
    departure_time: datetime.datetime | Unset = UNSET,
    destination: str | Unset = UNSET,
    arrival_time: datetime.datetime | Unset = UNSET,
    routing_mode: RoutingMode | Unset = UNSET,
    shape: Shape | Unset = UNSET,
    optimize_for: CalculateIsolinesPostOptimizeFor
    | Unset = CalculateIsolinesPostOptimizeFor.BALANCED,
    avoid: Avoid | Unset = UNSET,
    exclude: Exclude | Unset = UNSET,
    allow: Allow | Unset = UNSET,
    truck: Truck | Unset = UNSET,
    vehicle: Vehicle | Unset = UNSET,
    consumption_model: str | Unset = UNSET,
    ev: BaseEVEmpiricalModel | BaseEVPhysicalModel | Unset = UNSET,
    fuel: Fuel | Unset = UNSET,
    max_speed_on_segment: str | Unset = UNSET,
    taxi: Taxi | Unset = UNSET,
    traffic: Traffic | Unset = UNSET,
    billing_tag: str | Unset = UNSET,
    x_request_id: str | Unset = UNSET,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}
    if not isinstance(x_request_id, Unset):
        headers["X-Request-ID"] = x_request_id

    params: dict[str, Any] = {}

    json_range_ = range_.to_dict()
    params.update((f"range[{k}]", v) for k, v in json_range_.items())

    json_transport_mode = transport_mode.value
    params["transportMode"] = json_transport_mode

    params["origin"] = origin

    json_departure_time: str | Unset = UNSET
    if not isinstance(departure_time, Unset):
        json_departure_time = departure_time.isoformat()
    params["departureTime"] = json_departure_time

    params["destination"] = destination

    json_arrival_time: str | Unset = UNSET
    if not isinstance(arrival_time, Unset):
        json_arrival_time = arrival_time.isoformat()
    params["arrivalTime"] = json_arrival_time

    json_routing_mode: str | Unset = UNSET
    if not isinstance(routing_mode, Unset):
        json_routing_mode = routing_mode.value

    params["routingMode"] = json_routing_mode

    json_shape: dict[str, Any] | Unset = UNSET
    if not isinstance(shape, Unset):
        json_shape = shape.to_dict()
    if not isinstance(json_shape, Unset):
        params.update((f"shape[{k}]", v) for k, v in json_shape.items())

    json_optimize_for: str | Unset = UNSET
    if not isinstance(optimize_for, Unset):
        json_optimize_for = optimize_for.value

    params["optimizeFor"] = json_optimize_for

    json_avoid: dict[str, Any] | Unset = UNSET
    if not isinstance(avoid, Unset):
        json_avoid = avoid.to_dict()
    if not isinstance(json_avoid, Unset):
        params.update((f"avoid[{k}]", v) for k, v in json_avoid.items())

    json_exclude: dict[str, Any] | Unset = UNSET
    if not isinstance(exclude, Unset):
        json_exclude = exclude.to_dict()
    if not isinstance(json_exclude, Unset):
        params.update((f"exclude[{k}]", v) for k, v in json_exclude.items())

    json_allow: dict[str, Any] | Unset = UNSET
    if not isinstance(allow, Unset):
        json_allow = allow.to_dict()
    if not isinstance(json_allow, Unset):
        params.update((f"allow[{k}]", v) for k, v in json_allow.items())

    json_truck: dict[str, Any] | Unset = UNSET
    if not isinstance(truck, Unset):
        json_truck = truck.to_dict()
    if not isinstance(json_truck, Unset):
        params.update((f"truck[{k}]", v) for k, v in json_truck.items())

    json_vehicle: dict[str, Any] | Unset = UNSET
    if not isinstance(vehicle, Unset):
        json_vehicle = vehicle.to_dict()
    if not isinstance(json_vehicle, Unset):
        params.update((f"vehicle[{k}]", v) for k, v in json_vehicle.items())

    params["consumptionModel"] = consumption_model

    json_ev: dict[str, Any] | Unset
    if isinstance(ev, Unset):
        json_ev = UNSET
    elif isinstance(ev, BaseEVEmpiricalModel):
        json_ev = ev.to_dict()
    else:
        json_ev = ev.to_dict()

    params["ev"] = json_ev

    json_fuel: dict[str, Any] | Unset = UNSET
    if not isinstance(fuel, Unset):
        json_fuel = fuel.to_dict()
    if not isinstance(json_fuel, Unset):
        params.update((f"fuel[{k}]", v) for k, v in json_fuel.items())

    params["maxSpeedOnSegment"] = max_speed_on_segment

    json_taxi: dict[str, Any] | Unset = UNSET
    if not isinstance(taxi, Unset):
        json_taxi = taxi.to_dict()
    if not isinstance(json_taxi, Unset):
        params.update((f"taxi[{k}]", v) for k, v in json_taxi.items())

    json_traffic: dict[str, Any] | Unset = UNSET
    if not isinstance(traffic, Unset):
        json_traffic = traffic.to_dict()
    if not isinstance(json_traffic, Unset):
        params.update((f"traffic[{k}]", v) for k, v in json_traffic.items())

    params["billingTag"] = billing_tag

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/isolines",
        "params": params,
    }

    if not isinstance(body, Unset):
        _kwargs["json"] = body.to_dict()

    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> (
    AuthErrorResponseSchema
    | AuthErrorResponseSchema
    | NotAllowed
    | IsolineErrorResponse
    | IsolineResponse
    | ServerInternal
    | None
):
    if response.status_code == 200:
        response_200 = IsolineResponse.from_dict(response.json())

        return response_200

    if response.status_code == 400:
        response_400 = IsolineErrorResponse.from_dict(response.json())

        return response_400

    if response.status_code == 401:
        response_401 = AuthErrorResponseSchema.from_dict(response.json())

        return response_401

    if response.status_code == 403:

        def _parse_response_403(data: object) -> AuthErrorResponseSchema | NotAllowed:
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                componentsschemas_isoline_403_error_response_schema_type_0 = (
                    AuthErrorResponseSchema.from_dict(data)
                )

                return componentsschemas_isoline_403_error_response_schema_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            if not isinstance(data, dict):
                raise TypeError()
            componentsschemas_isoline_403_error_response_schema_type_1 = (
                NotAllowed.from_dict(data)
            )

            return componentsschemas_isoline_403_error_response_schema_type_1

        response_403 = _parse_response_403(response.json())

        return response_403

    if response.status_code == 500:
        response_500 = ServerInternal.from_dict(response.json())

        return response_500

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[
    AuthErrorResponseSchema
    | AuthErrorResponseSchema
    | NotAllowed
    | IsolineErrorResponse
    | IsolineResponse
    | ServerInternal
]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: CalculateIsolinesPostParameters | Unset = UNSET,
    range_: Range,
    transport_mode: RouterMode,
    origin: str | Unset = UNSET,
    departure_time: datetime.datetime | Unset = UNSET,
    destination: str | Unset = UNSET,
    arrival_time: datetime.datetime | Unset = UNSET,
    routing_mode: RoutingMode | Unset = UNSET,
    shape: Shape | Unset = UNSET,
    optimize_for: CalculateIsolinesPostOptimizeFor
    | Unset = CalculateIsolinesPostOptimizeFor.BALANCED,
    avoid: Avoid | Unset = UNSET,
    exclude: Exclude | Unset = UNSET,
    allow: Allow | Unset = UNSET,
    truck: Truck | Unset = UNSET,
    vehicle: Vehicle | Unset = UNSET,
    consumption_model: str | Unset = UNSET,
    ev: BaseEVEmpiricalModel | BaseEVPhysicalModel | Unset = UNSET,
    fuel: Fuel | Unset = UNSET,
    max_speed_on_segment: str | Unset = UNSET,
    taxi: Taxi | Unset = UNSET,
    traffic: Traffic | Unset = UNSET,
    billing_tag: str | Unset = UNSET,
    x_request_id: str | Unset = UNSET,
) -> Response[
    AuthErrorResponseSchema
    | AuthErrorResponseSchema
    | NotAllowed
    | IsolineErrorResponse
    | IsolineResponse
    | ServerInternal
]:
    """Calculate isolines via POST

     Retrieves a list of polylines that captures the approximate reachable area
    from the specified center point constrained by the provided parameters.

    At the moment, only select parameters are permitted in the POST payload. In particular, those
    parameters
    that due to request size may be limited in the query string. See the request body section below.
    These
    parameters can be provided either in the query string or in the POST body.
    All other parameters can only be provided in the query string at the moment.

    Args:
        range_ (Range): Ranges specified in terms of distance, travel time or energy consumption.
        transport_mode (RouterMode): Mode of transport to be used for route calculation.
        origin (str | Unset): A location defining an origin, destination or via point for a route
            or an isoline.

            Format: `Place[WaypointOptions]`

            * Place: `{lat},{lng}[PlaceOptions]`
            * PlaceOptions: `;option1=value1;option2=value2...`
            * WaypointOptions: `!option1=value1!option2=value2...`

            A waypoint consists of:
            * Exactly one place
            * Optional settings for the place
            * Optional settings for the waypoint itself

            Supported place options:
            * `course`: int, degrees clock-wise from north. Indicates the desired direction from the
            place. For example, `90` indicating `east`. Often combined with `radius` and/or
            `minCourseDistance`. This parameter takes preference over `matchSideOfStreet`.
            * `sideOfStreetHint`: `{lat},{lng}`. Indicates the side of the street that should be used.
            For example, if the location is to the left of the street, the router will prefer using
            that side in case the street has dividers. For example,
            `52.511496,13.304140;sideOfStreetHint=52.512149,13.304076` indicates that the `north` side
            of the street should be preferred. This options is required, if `matchSideOfStreet` is set
            to `always`. Option cannot be combined with `radius`, `radiusPenalty` or `snapRadius`.
            * `displayLocation`: `{lat},{lng}`. Indicates the physical location of the POI. It is
            different from the originalLocation and location which are generally expected to be on the
            navigable road network, i.e., on segments.
            * `uTurnPermission`: enum `[allow, avoid]`. Specifies the U-Turn Permission mode at the
            stop-over waypoint. If unspecified, the permission will be determined by the global
            setting, avoid[features]=uTurns. This feature is not supported for pass-through waypoints
            and U-Turns are generally avoided in that case.
              + `allow` : Allow making a U-Turn at this stop-over waypoint
              + `avoid` : Avoid making a U-Turn at this stop-over waypoint
            * `matchSideOfStreet`: enum `[always, onlyIfDivided]`. Specifies how the location set by
            `sideOfStreetHint` should be handled. Requires `sideOfStreetHint` to be specified as well.
            Note the exception above when combined with `course`.
              + `always` : Always prefer the given side of street.
              + `onlyIfDivided`: Only prefer using side of street set by `sideOfStreetHint` in case
            the street has dividers. This is the default behavior.
            * `nameHint`: string. Causes the router to look for the place with the most similar name.
            The typical examples include: `North` being used to differentiate between interstates `I66
            North` and `I66 South`, `Downtown Avenue` being used to correctly select a residental
            street.
            * `radius`: int, meters. Sets the radius within which all locations are considered equally
            eligible for selection as waypoints. Typical use cases include scenarios with imprecise
            coordinates or user input provided on low resolution map displays. The value is capped at
            200 meters. Setting a higher value doesn't return an error. Can't be used with
            `snapRadius`.
            * `snapRadius`: int, meters. Sets the radius within which waypoints are matched to the
            most "significant" place. Candidates for matching are sorted in the order of significance
            which is based on the visibility on a zoomed-out map. A highway is considered more
            significant than a national road, while a national road is more significant than a city
            road. Typical use case - selecting waypoints on a  zoomed-out view of a map on a drag-and-
            drop interface, where only significant roads are visible. A big enough radius allows to
            match waypoints to such roads. Can't be used with `radius` or `radiusPenalty` parameters.
            * `radiusPenalty`: int, percentage 0-10000. Penalty as percentage: Used in conjunction
            with the `radius` parameter. Router will match the waypoint within the specified radius
            and apply a penalty to candidates based on their air distance to the waypoint. This
            penalty is proportional to the given percentage, where 100 is just the cost of the air
            distance, and 200 is twice the cost of the air distance. The penalty must be chosen so
            that, when multiplied by the radius, the result is less than or equal to 7200. Regardless,
            only values up to and including 10000 will be accepted. This means that a maximum penalty
            of 3600% is allowed for a radius of 200m, 7200% for 100m and 10000% for 72m and less.
            Higher values will result in an error response. `radiusPenalty` cannot be combined with
            `snapRadius`. **Alpha**: This parameter is in development. It may not be stable and is
            subject to change.

            * `minCourseDistance`: int, meters. Instructs the routing service to try to find a route
            that avoids actions for the indicated distance. For example, if the origin is determined
            by a moving vehicle, the user might not have time to react to early actions. Values
            greater than 2000 meters will be capped at 2000 meters.
            * `segmentIdHint`: string. Causes the router to try and match to the specified segment.
            Waypoint coordinates need to be on the segment, otherwise waypoint will be matched
            ignoring the segment hint. This parameter can be used when the waypoint is too close to
            more than one segment to force matching to a specific one.
            * `onRoadThreshold`: int, meters. allows specifying a distance within which the waypoint
            could be considered as being on a highway/bridge/tunnel/sliproad. Within this threshold,
            the attributes of the segments do not impact the matching. Outside the threshold only
            segments which aren't one of highway/bridge/tunnel/sliproad can be matched.

            Supported waypoint options:
            * `stopDuration`: desired duration for the stop, in seconds.
               `stopDuration` must be less than 50000.
            * `passThrough`: boolean. Asks the router to avoid the following during route calculation:
              + Introducing a stop at the waypoint.
              + Splitting the route into sections.
              + Changing the direction of travel.

            Following scenarios are not supported for `passThrough` parameter:
              + Setting both `stopDuration` to a value greater than 0 and `passThrough=true`.
              + Setting `passThrough=true` for `origin` or `destination` of a route.
              The default value is `false`.
        departure_time (datetime.datetime | Unset): **RFC 3339**, section 5.6 as defined by either
            `date-time` or `date-only` 'T' `partial-time` (ie no time-offset).
        destination (str | Unset): A location defining an origin, destination or via point for a
            route or an isoline.

            Format: `Place[WaypointOptions]`

            * Place: `{lat},{lng}[PlaceOptions]`
            * PlaceOptions: `;option1=value1;option2=value2...`
            * WaypointOptions: `!option1=value1!option2=value2...`

            A waypoint consists of:
            * Exactly one place
            * Optional settings for the place
            * Optional settings for the waypoint itself

            Supported place options:
            * `course`: int, degrees clock-wise from north. Indicates the desired direction from the
            place. For example, `90` indicating `east`. Often combined with `radius` and/or
            `minCourseDistance`. This parameter takes preference over `matchSideOfStreet`.
            * `sideOfStreetHint`: `{lat},{lng}`. Indicates the side of the street that should be used.
            For example, if the location is to the left of the street, the router will prefer using
            that side in case the street has dividers. For example,
            `52.511496,13.304140;sideOfStreetHint=52.512149,13.304076` indicates that the `north` side
            of the street should be preferred. This options is required, if `matchSideOfStreet` is set
            to `always`. Option cannot be combined with `radius`, `radiusPenalty` or `snapRadius`.
            * `displayLocation`: `{lat},{lng}`. Indicates the physical location of the POI. It is
            different from the originalLocation and location which are generally expected to be on the
            navigable road network, i.e., on segments.
            * `uTurnPermission`: enum `[allow, avoid]`. Specifies the U-Turn Permission mode at the
            stop-over waypoint. If unspecified, the permission will be determined by the global
            setting, avoid[features]=uTurns. This feature is not supported for pass-through waypoints
            and U-Turns are generally avoided in that case.
              + `allow` : Allow making a U-Turn at this stop-over waypoint
              + `avoid` : Avoid making a U-Turn at this stop-over waypoint
            * `matchSideOfStreet`: enum `[always, onlyIfDivided]`. Specifies how the location set by
            `sideOfStreetHint` should be handled. Requires `sideOfStreetHint` to be specified as well.
            Note the exception above when combined with `course`.
              + `always` : Always prefer the given side of street.
              + `onlyIfDivided`: Only prefer using side of street set by `sideOfStreetHint` in case
            the street has dividers. This is the default behavior.
            * `nameHint`: string. Causes the router to look for the place with the most similar name.
            The typical examples include: `North` being used to differentiate between interstates `I66
            North` and `I66 South`, `Downtown Avenue` being used to correctly select a residental
            street.
            * `radius`: int, meters. Sets the radius within which all locations are considered equally
            eligible for selection as waypoints. Typical use cases include scenarios with imprecise
            coordinates or user input provided on low resolution map displays. The value is capped at
            200 meters. Setting a higher value doesn't return an error. Can't be used with
            `snapRadius`.
            * `snapRadius`: int, meters. Sets the radius within which waypoints are matched to the
            most "significant" place. Candidates for matching are sorted in the order of significance
            which is based on the visibility on a zoomed-out map. A highway is considered more
            significant than a national road, while a national road is more significant than a city
            road. Typical use case - selecting waypoints on a  zoomed-out view of a map on a drag-and-
            drop interface, where only significant roads are visible. A big enough radius allows to
            match waypoints to such roads. Can't be used with `radius` or `radiusPenalty` parameters.
            * `radiusPenalty`: int, percentage 0-10000. Penalty as percentage: Used in conjunction
            with the `radius` parameter. Router will match the waypoint within the specified radius
            and apply a penalty to candidates based on their air distance to the waypoint. This
            penalty is proportional to the given percentage, where 100 is just the cost of the air
            distance, and 200 is twice the cost of the air distance. The penalty must be chosen so
            that, when multiplied by the radius, the result is less than or equal to 7200. Regardless,
            only values up to and including 10000 will be accepted. This means that a maximum penalty
            of 3600% is allowed for a radius of 200m, 7200% for 100m and 10000% for 72m and less.
            Higher values will result in an error response. `radiusPenalty` cannot be combined with
            `snapRadius`. **Alpha**: This parameter is in development. It may not be stable and is
            subject to change.

            * `minCourseDistance`: int, meters. Instructs the routing service to try to find a route
            that avoids actions for the indicated distance. For example, if the origin is determined
            by a moving vehicle, the user might not have time to react to early actions. Values
            greater than 2000 meters will be capped at 2000 meters.
            * `segmentIdHint`: string. Causes the router to try and match to the specified segment.
            Waypoint coordinates need to be on the segment, otherwise waypoint will be matched
            ignoring the segment hint. This parameter can be used when the waypoint is too close to
            more than one segment to force matching to a specific one.
            * `onRoadThreshold`: int, meters. allows specifying a distance within which the waypoint
            could be considered as being on a highway/bridge/tunnel/sliproad. Within this threshold,
            the attributes of the segments do not impact the matching. Outside the threshold only
            segments which aren't one of highway/bridge/tunnel/sliproad can be matched.

            Supported waypoint options:
            * `stopDuration`: desired duration for the stop, in seconds.
               `stopDuration` must be less than 50000.
            * `passThrough`: boolean. Asks the router to avoid the following during route calculation:
              + Introducing a stop at the waypoint.
              + Splitting the route into sections.
              + Changing the direction of travel.

            Following scenarios are not supported for `passThrough` parameter:
              + Setting both `stopDuration` to a value greater than 0 and `passThrough=true`.
              + Setting `passThrough=true` for `origin` or `destination` of a route.
              The default value is `false`.
        arrival_time (datetime.datetime | Unset): **RFC 3339**, section 5.6 as defined by either
            `date-time` or `date-only` 'T' `partial-time` (ie no time-offset).
        routing_mode (RoutingMode | Unset): Specifies which optimization is applied during the
            calculation.

            * `fast`: Route calculation from start to destination optimized by travel time. In many
              cases, the route returned by `fast` mode may not be the route with the fastest
              possible travel time. For example, the routing service may favor a route that remains on
              a highway, even if a faster travel time can be achieved by taking a detour or shortcut
              through an inconvenient side road.
            * `short`: Route calculation from start to destination disregarding any speed information.
              In this mode, the distance of the route is minimized, while keeping the route sensible.
              This includes, for example, penalizing turns. Because of that, the resulting route will
              not necessarily be the one with minimal distance.

            Notes:
            * The following Transport modes only support `fast` routingMode
              - `bicycle`
              - `bus`
              - `pedestrian`
              - `privateBus`
              - `scooter`
              - `taxi`
        shape (Shape | Unset): A struct used to control the shape of the returned isolines.
        optimize_for (CalculateIsolinesPostOptimizeFor | Unset):  Default:
            CalculateIsolinesPostOptimizeFor.BALANCED.
        avoid (Avoid | Unset):
        exclude (Exclude | Unset): Options to exclude strictly during the route calculation.
        allow (Allow | Unset): Explicitly allow features that require users to opt in.
        truck (Truck | Unset): Vehicle-specific parameters
        vehicle (Vehicle | Unset): Vehicle-specific parameters
        consumption_model (str | Unset): Extensible enum: `empirical` `physical` `...`
            Specifies which of the EV consumption models is being used. See the `ev` parameter for
            details on the models.

            * `empirical`
            * `physical`

            **Alpha**: Physical consumption model is in development. It may not be stable and is
            subject to change.
        ev (BaseEVEmpiricalModel | BaseEVPhysicalModel | Unset): EV parameters to be used for
            calculating consumption and for calculating isolines for EV.

            There are two consumption models (`empirical`, `physical`). Set the `consumptionModel`
            parameter to choose which model to use.

            Required `empirical` model properties:
            * `freeFlowSpeedTable`

            Required `physical` model properties:
            * `driveEfficiency`
            * `recuperationEfficiency`

            When using the physical model, certain properties of the `vehicle` parameter are also
            required (see documentation for the `vehicle` parameter for more details):
            * `rollingResistanceCoefficient`
            * `airDragCoefficient`
            * `currentWeight`
            * `frontalArea` or combination of `width` and `height`
        fuel (Fuel | Unset): Fuel parameters to be used for calculating consumption and
            consumption-based isolines for fuel-based vehicles
            using empirical consumption model.

            Attributes:
            * `type`
            * `freeFlowSpeedTable`
            * `trafficSpeedTable`
            * `additionalConsumption`
            * `ascent`
        max_speed_on_segment (str | Unset): A comma separated list of segments with restrictions
            on maximum baseSpeed.

            Each entry has the following structure:
            `{segmentId}(#{direction})?;speed={maxBaseSpeed}`

            The individual parts are:
            * segmentId: The identifier of the referenced topology segment inside the catalog,
            example: `here:cm:segment:207551710`
            * direction (optional): Either '*' for bidirectional (default), '+' for positive
            direction, or '-' for negative direction
            * maxBaseSpeed: New value in m/s of baseSpeed on segment

            Example of a parameter value excluding two segments:
            `here:cm:segment:207551710#+;speed=10,here:cm:segment:76771992;speed=1`

            **Notes**:
            - It does not increase default baseSpeed on segment. If the value is greater than the
            default base speed, then such penalty will have no effect.
            - Minimum valid value for speed is 1
            - Using segments with a modified base speed does not trigger any notifications
            - Maximum number of penalized segments in one request cannot be greater than 1000.
              "penalized segments" refer to segments that have a restrictions on maximum baseSpeed
            with `maxSpeedOnSegment`
              or avoided with `avoid[segments]`
            - In case the same segment is penalized multiple times through values provided in the
            query string and/or the POST body,
              then the most restrictive value will be applied.
        taxi (Taxi | Unset): Taxi-specific parameters
        traffic (Traffic | Unset): Traffic specific parameters.
        billing_tag (str | Unset):  Example: ABCD+EFGH.
        x_request_id (str | Unset):
        body (CalculateIsolinesPostParameters | Unset): Parameters of the POST body for isoline
            calculation

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[AuthErrorResponseSchema | AuthErrorResponseSchema | NotAllowed | IsolineErrorResponse | IsolineResponse | ServerInternal]
    """

    kwargs = _get_kwargs(
        body=body,
        range_=range_,
        transport_mode=transport_mode,
        origin=origin,
        departure_time=departure_time,
        destination=destination,
        arrival_time=arrival_time,
        routing_mode=routing_mode,
        shape=shape,
        optimize_for=optimize_for,
        avoid=avoid,
        exclude=exclude,
        allow=allow,
        truck=truck,
        vehicle=vehicle,
        consumption_model=consumption_model,
        ev=ev,
        fuel=fuel,
        max_speed_on_segment=max_speed_on_segment,
        taxi=taxi,
        traffic=traffic,
        billing_tag=billing_tag,
        x_request_id=x_request_id,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient | Client,
    body: CalculateIsolinesPostParameters | Unset = UNSET,
    range_: Range,
    transport_mode: RouterMode,
    origin: str | Unset = UNSET,
    departure_time: datetime.datetime | Unset = UNSET,
    destination: str | Unset = UNSET,
    arrival_time: datetime.datetime | Unset = UNSET,
    routing_mode: RoutingMode | Unset = UNSET,
    shape: Shape | Unset = UNSET,
    optimize_for: CalculateIsolinesPostOptimizeFor
    | Unset = CalculateIsolinesPostOptimizeFor.BALANCED,
    avoid: Avoid | Unset = UNSET,
    exclude: Exclude | Unset = UNSET,
    allow: Allow | Unset = UNSET,
    truck: Truck | Unset = UNSET,
    vehicle: Vehicle | Unset = UNSET,
    consumption_model: str | Unset = UNSET,
    ev: BaseEVEmpiricalModel | BaseEVPhysicalModel | Unset = UNSET,
    fuel: Fuel | Unset = UNSET,
    max_speed_on_segment: str | Unset = UNSET,
    taxi: Taxi | Unset = UNSET,
    traffic: Traffic | Unset = UNSET,
    billing_tag: str | Unset = UNSET,
    x_request_id: str | Unset = UNSET,
) -> (
    AuthErrorResponseSchema
    | AuthErrorResponseSchema
    | NotAllowed
    | IsolineErrorResponse
    | IsolineResponse
    | ServerInternal
    | None
):
    """Calculate isolines via POST

     Retrieves a list of polylines that captures the approximate reachable area
    from the specified center point constrained by the provided parameters.

    At the moment, only select parameters are permitted in the POST payload. In particular, those
    parameters
    that due to request size may be limited in the query string. See the request body section below.
    These
    parameters can be provided either in the query string or in the POST body.
    All other parameters can only be provided in the query string at the moment.

    Args:
        range_ (Range): Ranges specified in terms of distance, travel time or energy consumption.
        transport_mode (RouterMode): Mode of transport to be used for route calculation.
        origin (str | Unset): A location defining an origin, destination or via point for a route
            or an isoline.

            Format: `Place[WaypointOptions]`

            * Place: `{lat},{lng}[PlaceOptions]`
            * PlaceOptions: `;option1=value1;option2=value2...`
            * WaypointOptions: `!option1=value1!option2=value2...`

            A waypoint consists of:
            * Exactly one place
            * Optional settings for the place
            * Optional settings for the waypoint itself

            Supported place options:
            * `course`: int, degrees clock-wise from north. Indicates the desired direction from the
            place. For example, `90` indicating `east`. Often combined with `radius` and/or
            `minCourseDistance`. This parameter takes preference over `matchSideOfStreet`.
            * `sideOfStreetHint`: `{lat},{lng}`. Indicates the side of the street that should be used.
            For example, if the location is to the left of the street, the router will prefer using
            that side in case the street has dividers. For example,
            `52.511496,13.304140;sideOfStreetHint=52.512149,13.304076` indicates that the `north` side
            of the street should be preferred. This options is required, if `matchSideOfStreet` is set
            to `always`. Option cannot be combined with `radius`, `radiusPenalty` or `snapRadius`.
            * `displayLocation`: `{lat},{lng}`. Indicates the physical location of the POI. It is
            different from the originalLocation and location which are generally expected to be on the
            navigable road network, i.e., on segments.
            * `uTurnPermission`: enum `[allow, avoid]`. Specifies the U-Turn Permission mode at the
            stop-over waypoint. If unspecified, the permission will be determined by the global
            setting, avoid[features]=uTurns. This feature is not supported for pass-through waypoints
            and U-Turns are generally avoided in that case.
              + `allow` : Allow making a U-Turn at this stop-over waypoint
              + `avoid` : Avoid making a U-Turn at this stop-over waypoint
            * `matchSideOfStreet`: enum `[always, onlyIfDivided]`. Specifies how the location set by
            `sideOfStreetHint` should be handled. Requires `sideOfStreetHint` to be specified as well.
            Note the exception above when combined with `course`.
              + `always` : Always prefer the given side of street.
              + `onlyIfDivided`: Only prefer using side of street set by `sideOfStreetHint` in case
            the street has dividers. This is the default behavior.
            * `nameHint`: string. Causes the router to look for the place with the most similar name.
            The typical examples include: `North` being used to differentiate between interstates `I66
            North` and `I66 South`, `Downtown Avenue` being used to correctly select a residental
            street.
            * `radius`: int, meters. Sets the radius within which all locations are considered equally
            eligible for selection as waypoints. Typical use cases include scenarios with imprecise
            coordinates or user input provided on low resolution map displays. The value is capped at
            200 meters. Setting a higher value doesn't return an error. Can't be used with
            `snapRadius`.
            * `snapRadius`: int, meters. Sets the radius within which waypoints are matched to the
            most "significant" place. Candidates for matching are sorted in the order of significance
            which is based on the visibility on a zoomed-out map. A highway is considered more
            significant than a national road, while a national road is more significant than a city
            road. Typical use case - selecting waypoints on a  zoomed-out view of a map on a drag-and-
            drop interface, where only significant roads are visible. A big enough radius allows to
            match waypoints to such roads. Can't be used with `radius` or `radiusPenalty` parameters.
            * `radiusPenalty`: int, percentage 0-10000. Penalty as percentage: Used in conjunction
            with the `radius` parameter. Router will match the waypoint within the specified radius
            and apply a penalty to candidates based on their air distance to the waypoint. This
            penalty is proportional to the given percentage, where 100 is just the cost of the air
            distance, and 200 is twice the cost of the air distance. The penalty must be chosen so
            that, when multiplied by the radius, the result is less than or equal to 7200. Regardless,
            only values up to and including 10000 will be accepted. This means that a maximum penalty
            of 3600% is allowed for a radius of 200m, 7200% for 100m and 10000% for 72m and less.
            Higher values will result in an error response. `radiusPenalty` cannot be combined with
            `snapRadius`. **Alpha**: This parameter is in development. It may not be stable and is
            subject to change.

            * `minCourseDistance`: int, meters. Instructs the routing service to try to find a route
            that avoids actions for the indicated distance. For example, if the origin is determined
            by a moving vehicle, the user might not have time to react to early actions. Values
            greater than 2000 meters will be capped at 2000 meters.
            * `segmentIdHint`: string. Causes the router to try and match to the specified segment.
            Waypoint coordinates need to be on the segment, otherwise waypoint will be matched
            ignoring the segment hint. This parameter can be used when the waypoint is too close to
            more than one segment to force matching to a specific one.
            * `onRoadThreshold`: int, meters. allows specifying a distance within which the waypoint
            could be considered as being on a highway/bridge/tunnel/sliproad. Within this threshold,
            the attributes of the segments do not impact the matching. Outside the threshold only
            segments which aren't one of highway/bridge/tunnel/sliproad can be matched.

            Supported waypoint options:
            * `stopDuration`: desired duration for the stop, in seconds.
               `stopDuration` must be less than 50000.
            * `passThrough`: boolean. Asks the router to avoid the following during route calculation:
              + Introducing a stop at the waypoint.
              + Splitting the route into sections.
              + Changing the direction of travel.

            Following scenarios are not supported for `passThrough` parameter:
              + Setting both `stopDuration` to a value greater than 0 and `passThrough=true`.
              + Setting `passThrough=true` for `origin` or `destination` of a route.
              The default value is `false`.
        departure_time (datetime.datetime | Unset): **RFC 3339**, section 5.6 as defined by either
            `date-time` or `date-only` 'T' `partial-time` (ie no time-offset).
        destination (str | Unset): A location defining an origin, destination or via point for a
            route or an isoline.

            Format: `Place[WaypointOptions]`

            * Place: `{lat},{lng}[PlaceOptions]`
            * PlaceOptions: `;option1=value1;option2=value2...`
            * WaypointOptions: `!option1=value1!option2=value2...`

            A waypoint consists of:
            * Exactly one place
            * Optional settings for the place
            * Optional settings for the waypoint itself

            Supported place options:
            * `course`: int, degrees clock-wise from north. Indicates the desired direction from the
            place. For example, `90` indicating `east`. Often combined with `radius` and/or
            `minCourseDistance`. This parameter takes preference over `matchSideOfStreet`.
            * `sideOfStreetHint`: `{lat},{lng}`. Indicates the side of the street that should be used.
            For example, if the location is to the left of the street, the router will prefer using
            that side in case the street has dividers. For example,
            `52.511496,13.304140;sideOfStreetHint=52.512149,13.304076` indicates that the `north` side
            of the street should be preferred. This options is required, if `matchSideOfStreet` is set
            to `always`. Option cannot be combined with `radius`, `radiusPenalty` or `snapRadius`.
            * `displayLocation`: `{lat},{lng}`. Indicates the physical location of the POI. It is
            different from the originalLocation and location which are generally expected to be on the
            navigable road network, i.e., on segments.
            * `uTurnPermission`: enum `[allow, avoid]`. Specifies the U-Turn Permission mode at the
            stop-over waypoint. If unspecified, the permission will be determined by the global
            setting, avoid[features]=uTurns. This feature is not supported for pass-through waypoints
            and U-Turns are generally avoided in that case.
              + `allow` : Allow making a U-Turn at this stop-over waypoint
              + `avoid` : Avoid making a U-Turn at this stop-over waypoint
            * `matchSideOfStreet`: enum `[always, onlyIfDivided]`. Specifies how the location set by
            `sideOfStreetHint` should be handled. Requires `sideOfStreetHint` to be specified as well.
            Note the exception above when combined with `course`.
              + `always` : Always prefer the given side of street.
              + `onlyIfDivided`: Only prefer using side of street set by `sideOfStreetHint` in case
            the street has dividers. This is the default behavior.
            * `nameHint`: string. Causes the router to look for the place with the most similar name.
            The typical examples include: `North` being used to differentiate between interstates `I66
            North` and `I66 South`, `Downtown Avenue` being used to correctly select a residental
            street.
            * `radius`: int, meters. Sets the radius within which all locations are considered equally
            eligible for selection as waypoints. Typical use cases include scenarios with imprecise
            coordinates or user input provided on low resolution map displays. The value is capped at
            200 meters. Setting a higher value doesn't return an error. Can't be used with
            `snapRadius`.
            * `snapRadius`: int, meters. Sets the radius within which waypoints are matched to the
            most "significant" place. Candidates for matching are sorted in the order of significance
            which is based on the visibility on a zoomed-out map. A highway is considered more
            significant than a national road, while a national road is more significant than a city
            road. Typical use case - selecting waypoints on a  zoomed-out view of a map on a drag-and-
            drop interface, where only significant roads are visible. A big enough radius allows to
            match waypoints to such roads. Can't be used with `radius` or `radiusPenalty` parameters.
            * `radiusPenalty`: int, percentage 0-10000. Penalty as percentage: Used in conjunction
            with the `radius` parameter. Router will match the waypoint within the specified radius
            and apply a penalty to candidates based on their air distance to the waypoint. This
            penalty is proportional to the given percentage, where 100 is just the cost of the air
            distance, and 200 is twice the cost of the air distance. The penalty must be chosen so
            that, when multiplied by the radius, the result is less than or equal to 7200. Regardless,
            only values up to and including 10000 will be accepted. This means that a maximum penalty
            of 3600% is allowed for a radius of 200m, 7200% for 100m and 10000% for 72m and less.
            Higher values will result in an error response. `radiusPenalty` cannot be combined with
            `snapRadius`. **Alpha**: This parameter is in development. It may not be stable and is
            subject to change.

            * `minCourseDistance`: int, meters. Instructs the routing service to try to find a route
            that avoids actions for the indicated distance. For example, if the origin is determined
            by a moving vehicle, the user might not have time to react to early actions. Values
            greater than 2000 meters will be capped at 2000 meters.
            * `segmentIdHint`: string. Causes the router to try and match to the specified segment.
            Waypoint coordinates need to be on the segment, otherwise waypoint will be matched
            ignoring the segment hint. This parameter can be used when the waypoint is too close to
            more than one segment to force matching to a specific one.
            * `onRoadThreshold`: int, meters. allows specifying a distance within which the waypoint
            could be considered as being on a highway/bridge/tunnel/sliproad. Within this threshold,
            the attributes of the segments do not impact the matching. Outside the threshold only
            segments which aren't one of highway/bridge/tunnel/sliproad can be matched.

            Supported waypoint options:
            * `stopDuration`: desired duration for the stop, in seconds.
               `stopDuration` must be less than 50000.
            * `passThrough`: boolean. Asks the router to avoid the following during route calculation:
              + Introducing a stop at the waypoint.
              + Splitting the route into sections.
              + Changing the direction of travel.

            Following scenarios are not supported for `passThrough` parameter:
              + Setting both `stopDuration` to a value greater than 0 and `passThrough=true`.
              + Setting `passThrough=true` for `origin` or `destination` of a route.
              The default value is `false`.
        arrival_time (datetime.datetime | Unset): **RFC 3339**, section 5.6 as defined by either
            `date-time` or `date-only` 'T' `partial-time` (ie no time-offset).
        routing_mode (RoutingMode | Unset): Specifies which optimization is applied during the
            calculation.

            * `fast`: Route calculation from start to destination optimized by travel time. In many
              cases, the route returned by `fast` mode may not be the route with the fastest
              possible travel time. For example, the routing service may favor a route that remains on
              a highway, even if a faster travel time can be achieved by taking a detour or shortcut
              through an inconvenient side road.
            * `short`: Route calculation from start to destination disregarding any speed information.
              In this mode, the distance of the route is minimized, while keeping the route sensible.
              This includes, for example, penalizing turns. Because of that, the resulting route will
              not necessarily be the one with minimal distance.

            Notes:
            * The following Transport modes only support `fast` routingMode
              - `bicycle`
              - `bus`
              - `pedestrian`
              - `privateBus`
              - `scooter`
              - `taxi`
        shape (Shape | Unset): A struct used to control the shape of the returned isolines.
        optimize_for (CalculateIsolinesPostOptimizeFor | Unset):  Default:
            CalculateIsolinesPostOptimizeFor.BALANCED.
        avoid (Avoid | Unset):
        exclude (Exclude | Unset): Options to exclude strictly during the route calculation.
        allow (Allow | Unset): Explicitly allow features that require users to opt in.
        truck (Truck | Unset): Vehicle-specific parameters
        vehicle (Vehicle | Unset): Vehicle-specific parameters
        consumption_model (str | Unset): Extensible enum: `empirical` `physical` `...`
            Specifies which of the EV consumption models is being used. See the `ev` parameter for
            details on the models.

            * `empirical`
            * `physical`

            **Alpha**: Physical consumption model is in development. It may not be stable and is
            subject to change.
        ev (BaseEVEmpiricalModel | BaseEVPhysicalModel | Unset): EV parameters to be used for
            calculating consumption and for calculating isolines for EV.

            There are two consumption models (`empirical`, `physical`). Set the `consumptionModel`
            parameter to choose which model to use.

            Required `empirical` model properties:
            * `freeFlowSpeedTable`

            Required `physical` model properties:
            * `driveEfficiency`
            * `recuperationEfficiency`

            When using the physical model, certain properties of the `vehicle` parameter are also
            required (see documentation for the `vehicle` parameter for more details):
            * `rollingResistanceCoefficient`
            * `airDragCoefficient`
            * `currentWeight`
            * `frontalArea` or combination of `width` and `height`
        fuel (Fuel | Unset): Fuel parameters to be used for calculating consumption and
            consumption-based isolines for fuel-based vehicles
            using empirical consumption model.

            Attributes:
            * `type`
            * `freeFlowSpeedTable`
            * `trafficSpeedTable`
            * `additionalConsumption`
            * `ascent`
        max_speed_on_segment (str | Unset): A comma separated list of segments with restrictions
            on maximum baseSpeed.

            Each entry has the following structure:
            `{segmentId}(#{direction})?;speed={maxBaseSpeed}`

            The individual parts are:
            * segmentId: The identifier of the referenced topology segment inside the catalog,
            example: `here:cm:segment:207551710`
            * direction (optional): Either '*' for bidirectional (default), '+' for positive
            direction, or '-' for negative direction
            * maxBaseSpeed: New value in m/s of baseSpeed on segment

            Example of a parameter value excluding two segments:
            `here:cm:segment:207551710#+;speed=10,here:cm:segment:76771992;speed=1`

            **Notes**:
            - It does not increase default baseSpeed on segment. If the value is greater than the
            default base speed, then such penalty will have no effect.
            - Minimum valid value for speed is 1
            - Using segments with a modified base speed does not trigger any notifications
            - Maximum number of penalized segments in one request cannot be greater than 1000.
              "penalized segments" refer to segments that have a restrictions on maximum baseSpeed
            with `maxSpeedOnSegment`
              or avoided with `avoid[segments]`
            - In case the same segment is penalized multiple times through values provided in the
            query string and/or the POST body,
              then the most restrictive value will be applied.
        taxi (Taxi | Unset): Taxi-specific parameters
        traffic (Traffic | Unset): Traffic specific parameters.
        billing_tag (str | Unset):  Example: ABCD+EFGH.
        x_request_id (str | Unset):
        body (CalculateIsolinesPostParameters | Unset): Parameters of the POST body for isoline
            calculation

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        AuthErrorResponseSchema | AuthErrorResponseSchema | NotAllowed | IsolineErrorResponse | IsolineResponse | ServerInternal
    """

    return sync_detailed(
        client=client,
        body=body,
        range_=range_,
        transport_mode=transport_mode,
        origin=origin,
        departure_time=departure_time,
        destination=destination,
        arrival_time=arrival_time,
        routing_mode=routing_mode,
        shape=shape,
        optimize_for=optimize_for,
        avoid=avoid,
        exclude=exclude,
        allow=allow,
        truck=truck,
        vehicle=vehicle,
        consumption_model=consumption_model,
        ev=ev,
        fuel=fuel,
        max_speed_on_segment=max_speed_on_segment,
        taxi=taxi,
        traffic=traffic,
        billing_tag=billing_tag,
        x_request_id=x_request_id,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient | Client,
    body: CalculateIsolinesPostParameters | Unset = UNSET,
    range_: Range,
    transport_mode: RouterMode,
    origin: str | Unset = UNSET,
    departure_time: datetime.datetime | Unset = UNSET,
    destination: str | Unset = UNSET,
    arrival_time: datetime.datetime | Unset = UNSET,
    routing_mode: RoutingMode | Unset = UNSET,
    shape: Shape | Unset = UNSET,
    optimize_for: CalculateIsolinesPostOptimizeFor
    | Unset = CalculateIsolinesPostOptimizeFor.BALANCED,
    avoid: Avoid | Unset = UNSET,
    exclude: Exclude | Unset = UNSET,
    allow: Allow | Unset = UNSET,
    truck: Truck | Unset = UNSET,
    vehicle: Vehicle | Unset = UNSET,
    consumption_model: str | Unset = UNSET,
    ev: BaseEVEmpiricalModel | BaseEVPhysicalModel | Unset = UNSET,
    fuel: Fuel | Unset = UNSET,
    max_speed_on_segment: str | Unset = UNSET,
    taxi: Taxi | Unset = UNSET,
    traffic: Traffic | Unset = UNSET,
    billing_tag: str | Unset = UNSET,
    x_request_id: str | Unset = UNSET,
) -> Response[
    AuthErrorResponseSchema
    | AuthErrorResponseSchema
    | NotAllowed
    | IsolineErrorResponse
    | IsolineResponse
    | ServerInternal
]:
    """Calculate isolines via POST

     Retrieves a list of polylines that captures the approximate reachable area
    from the specified center point constrained by the provided parameters.

    At the moment, only select parameters are permitted in the POST payload. In particular, those
    parameters
    that due to request size may be limited in the query string. See the request body section below.
    These
    parameters can be provided either in the query string or in the POST body.
    All other parameters can only be provided in the query string at the moment.

    Args:
        range_ (Range): Ranges specified in terms of distance, travel time or energy consumption.
        transport_mode (RouterMode): Mode of transport to be used for route calculation.
        origin (str | Unset): A location defining an origin, destination or via point for a route
            or an isoline.

            Format: `Place[WaypointOptions]`

            * Place: `{lat},{lng}[PlaceOptions]`
            * PlaceOptions: `;option1=value1;option2=value2...`
            * WaypointOptions: `!option1=value1!option2=value2...`

            A waypoint consists of:
            * Exactly one place
            * Optional settings for the place
            * Optional settings for the waypoint itself

            Supported place options:
            * `course`: int, degrees clock-wise from north. Indicates the desired direction from the
            place. For example, `90` indicating `east`. Often combined with `radius` and/or
            `minCourseDistance`. This parameter takes preference over `matchSideOfStreet`.
            * `sideOfStreetHint`: `{lat},{lng}`. Indicates the side of the street that should be used.
            For example, if the location is to the left of the street, the router will prefer using
            that side in case the street has dividers. For example,
            `52.511496,13.304140;sideOfStreetHint=52.512149,13.304076` indicates that the `north` side
            of the street should be preferred. This options is required, if `matchSideOfStreet` is set
            to `always`. Option cannot be combined with `radius`, `radiusPenalty` or `snapRadius`.
            * `displayLocation`: `{lat},{lng}`. Indicates the physical location of the POI. It is
            different from the originalLocation and location which are generally expected to be on the
            navigable road network, i.e., on segments.
            * `uTurnPermission`: enum `[allow, avoid]`. Specifies the U-Turn Permission mode at the
            stop-over waypoint. If unspecified, the permission will be determined by the global
            setting, avoid[features]=uTurns. This feature is not supported for pass-through waypoints
            and U-Turns are generally avoided in that case.
              + `allow` : Allow making a U-Turn at this stop-over waypoint
              + `avoid` : Avoid making a U-Turn at this stop-over waypoint
            * `matchSideOfStreet`: enum `[always, onlyIfDivided]`. Specifies how the location set by
            `sideOfStreetHint` should be handled. Requires `sideOfStreetHint` to be specified as well.
            Note the exception above when combined with `course`.
              + `always` : Always prefer the given side of street.
              + `onlyIfDivided`: Only prefer using side of street set by `sideOfStreetHint` in case
            the street has dividers. This is the default behavior.
            * `nameHint`: string. Causes the router to look for the place with the most similar name.
            The typical examples include: `North` being used to differentiate between interstates `I66
            North` and `I66 South`, `Downtown Avenue` being used to correctly select a residental
            street.
            * `radius`: int, meters. Sets the radius within which all locations are considered equally
            eligible for selection as waypoints. Typical use cases include scenarios with imprecise
            coordinates or user input provided on low resolution map displays. The value is capped at
            200 meters. Setting a higher value doesn't return an error. Can't be used with
            `snapRadius`.
            * `snapRadius`: int, meters. Sets the radius within which waypoints are matched to the
            most "significant" place. Candidates for matching are sorted in the order of significance
            which is based on the visibility on a zoomed-out map. A highway is considered more
            significant than a national road, while a national road is more significant than a city
            road. Typical use case - selecting waypoints on a  zoomed-out view of a map on a drag-and-
            drop interface, where only significant roads are visible. A big enough radius allows to
            match waypoints to such roads. Can't be used with `radius` or `radiusPenalty` parameters.
            * `radiusPenalty`: int, percentage 0-10000. Penalty as percentage: Used in conjunction
            with the `radius` parameter. Router will match the waypoint within the specified radius
            and apply a penalty to candidates based on their air distance to the waypoint. This
            penalty is proportional to the given percentage, where 100 is just the cost of the air
            distance, and 200 is twice the cost of the air distance. The penalty must be chosen so
            that, when multiplied by the radius, the result is less than or equal to 7200. Regardless,
            only values up to and including 10000 will be accepted. This means that a maximum penalty
            of 3600% is allowed for a radius of 200m, 7200% for 100m and 10000% for 72m and less.
            Higher values will result in an error response. `radiusPenalty` cannot be combined with
            `snapRadius`. **Alpha**: This parameter is in development. It may not be stable and is
            subject to change.

            * `minCourseDistance`: int, meters. Instructs the routing service to try to find a route
            that avoids actions for the indicated distance. For example, if the origin is determined
            by a moving vehicle, the user might not have time to react to early actions. Values
            greater than 2000 meters will be capped at 2000 meters.
            * `segmentIdHint`: string. Causes the router to try and match to the specified segment.
            Waypoint coordinates need to be on the segment, otherwise waypoint will be matched
            ignoring the segment hint. This parameter can be used when the waypoint is too close to
            more than one segment to force matching to a specific one.
            * `onRoadThreshold`: int, meters. allows specifying a distance within which the waypoint
            could be considered as being on a highway/bridge/tunnel/sliproad. Within this threshold,
            the attributes of the segments do not impact the matching. Outside the threshold only
            segments which aren't one of highway/bridge/tunnel/sliproad can be matched.

            Supported waypoint options:
            * `stopDuration`: desired duration for the stop, in seconds.
               `stopDuration` must be less than 50000.
            * `passThrough`: boolean. Asks the router to avoid the following during route calculation:
              + Introducing a stop at the waypoint.
              + Splitting the route into sections.
              + Changing the direction of travel.

            Following scenarios are not supported for `passThrough` parameter:
              + Setting both `stopDuration` to a value greater than 0 and `passThrough=true`.
              + Setting `passThrough=true` for `origin` or `destination` of a route.
              The default value is `false`.
        departure_time (datetime.datetime | Unset): **RFC 3339**, section 5.6 as defined by either
            `date-time` or `date-only` 'T' `partial-time` (ie no time-offset).
        destination (str | Unset): A location defining an origin, destination or via point for a
            route or an isoline.

            Format: `Place[WaypointOptions]`

            * Place: `{lat},{lng}[PlaceOptions]`
            * PlaceOptions: `;option1=value1;option2=value2...`
            * WaypointOptions: `!option1=value1!option2=value2...`

            A waypoint consists of:
            * Exactly one place
            * Optional settings for the place
            * Optional settings for the waypoint itself

            Supported place options:
            * `course`: int, degrees clock-wise from north. Indicates the desired direction from the
            place. For example, `90` indicating `east`. Often combined with `radius` and/or
            `minCourseDistance`. This parameter takes preference over `matchSideOfStreet`.
            * `sideOfStreetHint`: `{lat},{lng}`. Indicates the side of the street that should be used.
            For example, if the location is to the left of the street, the router will prefer using
            that side in case the street has dividers. For example,
            `52.511496,13.304140;sideOfStreetHint=52.512149,13.304076` indicates that the `north` side
            of the street should be preferred. This options is required, if `matchSideOfStreet` is set
            to `always`. Option cannot be combined with `radius`, `radiusPenalty` or `snapRadius`.
            * `displayLocation`: `{lat},{lng}`. Indicates the physical location of the POI. It is
            different from the originalLocation and location which are generally expected to be on the
            navigable road network, i.e., on segments.
            * `uTurnPermission`: enum `[allow, avoid]`. Specifies the U-Turn Permission mode at the
            stop-over waypoint. If unspecified, the permission will be determined by the global
            setting, avoid[features]=uTurns. This feature is not supported for pass-through waypoints
            and U-Turns are generally avoided in that case.
              + `allow` : Allow making a U-Turn at this stop-over waypoint
              + `avoid` : Avoid making a U-Turn at this stop-over waypoint
            * `matchSideOfStreet`: enum `[always, onlyIfDivided]`. Specifies how the location set by
            `sideOfStreetHint` should be handled. Requires `sideOfStreetHint` to be specified as well.
            Note the exception above when combined with `course`.
              + `always` : Always prefer the given side of street.
              + `onlyIfDivided`: Only prefer using side of street set by `sideOfStreetHint` in case
            the street has dividers. This is the default behavior.
            * `nameHint`: string. Causes the router to look for the place with the most similar name.
            The typical examples include: `North` being used to differentiate between interstates `I66
            North` and `I66 South`, `Downtown Avenue` being used to correctly select a residental
            street.
            * `radius`: int, meters. Sets the radius within which all locations are considered equally
            eligible for selection as waypoints. Typical use cases include scenarios with imprecise
            coordinates or user input provided on low resolution map displays. The value is capped at
            200 meters. Setting a higher value doesn't return an error. Can't be used with
            `snapRadius`.
            * `snapRadius`: int, meters. Sets the radius within which waypoints are matched to the
            most "significant" place. Candidates for matching are sorted in the order of significance
            which is based on the visibility on a zoomed-out map. A highway is considered more
            significant than a national road, while a national road is more significant than a city
            road. Typical use case - selecting waypoints on a  zoomed-out view of a map on a drag-and-
            drop interface, where only significant roads are visible. A big enough radius allows to
            match waypoints to such roads. Can't be used with `radius` or `radiusPenalty` parameters.
            * `radiusPenalty`: int, percentage 0-10000. Penalty as percentage: Used in conjunction
            with the `radius` parameter. Router will match the waypoint within the specified radius
            and apply a penalty to candidates based on their air distance to the waypoint. This
            penalty is proportional to the given percentage, where 100 is just the cost of the air
            distance, and 200 is twice the cost of the air distance. The penalty must be chosen so
            that, when multiplied by the radius, the result is less than or equal to 7200. Regardless,
            only values up to and including 10000 will be accepted. This means that a maximum penalty
            of 3600% is allowed for a radius of 200m, 7200% for 100m and 10000% for 72m and less.
            Higher values will result in an error response. `radiusPenalty` cannot be combined with
            `snapRadius`. **Alpha**: This parameter is in development. It may not be stable and is
            subject to change.

            * `minCourseDistance`: int, meters. Instructs the routing service to try to find a route
            that avoids actions for the indicated distance. For example, if the origin is determined
            by a moving vehicle, the user might not have time to react to early actions. Values
            greater than 2000 meters will be capped at 2000 meters.
            * `segmentIdHint`: string. Causes the router to try and match to the specified segment.
            Waypoint coordinates need to be on the segment, otherwise waypoint will be matched
            ignoring the segment hint. This parameter can be used when the waypoint is too close to
            more than one segment to force matching to a specific one.
            * `onRoadThreshold`: int, meters. allows specifying a distance within which the waypoint
            could be considered as being on a highway/bridge/tunnel/sliproad. Within this threshold,
            the attributes of the segments do not impact the matching. Outside the threshold only
            segments which aren't one of highway/bridge/tunnel/sliproad can be matched.

            Supported waypoint options:
            * `stopDuration`: desired duration for the stop, in seconds.
               `stopDuration` must be less than 50000.
            * `passThrough`: boolean. Asks the router to avoid the following during route calculation:
              + Introducing a stop at the waypoint.
              + Splitting the route into sections.
              + Changing the direction of travel.

            Following scenarios are not supported for `passThrough` parameter:
              + Setting both `stopDuration` to a value greater than 0 and `passThrough=true`.
              + Setting `passThrough=true` for `origin` or `destination` of a route.
              The default value is `false`.
        arrival_time (datetime.datetime | Unset): **RFC 3339**, section 5.6 as defined by either
            `date-time` or `date-only` 'T' `partial-time` (ie no time-offset).
        routing_mode (RoutingMode | Unset): Specifies which optimization is applied during the
            calculation.

            * `fast`: Route calculation from start to destination optimized by travel time. In many
              cases, the route returned by `fast` mode may not be the route with the fastest
              possible travel time. For example, the routing service may favor a route that remains on
              a highway, even if a faster travel time can be achieved by taking a detour or shortcut
              through an inconvenient side road.
            * `short`: Route calculation from start to destination disregarding any speed information.
              In this mode, the distance of the route is minimized, while keeping the route sensible.
              This includes, for example, penalizing turns. Because of that, the resulting route will
              not necessarily be the one with minimal distance.

            Notes:
            * The following Transport modes only support `fast` routingMode
              - `bicycle`
              - `bus`
              - `pedestrian`
              - `privateBus`
              - `scooter`
              - `taxi`
        shape (Shape | Unset): A struct used to control the shape of the returned isolines.
        optimize_for (CalculateIsolinesPostOptimizeFor | Unset):  Default:
            CalculateIsolinesPostOptimizeFor.BALANCED.
        avoid (Avoid | Unset):
        exclude (Exclude | Unset): Options to exclude strictly during the route calculation.
        allow (Allow | Unset): Explicitly allow features that require users to opt in.
        truck (Truck | Unset): Vehicle-specific parameters
        vehicle (Vehicle | Unset): Vehicle-specific parameters
        consumption_model (str | Unset): Extensible enum: `empirical` `physical` `...`
            Specifies which of the EV consumption models is being used. See the `ev` parameter for
            details on the models.

            * `empirical`
            * `physical`

            **Alpha**: Physical consumption model is in development. It may not be stable and is
            subject to change.
        ev (BaseEVEmpiricalModel | BaseEVPhysicalModel | Unset): EV parameters to be used for
            calculating consumption and for calculating isolines for EV.

            There are two consumption models (`empirical`, `physical`). Set the `consumptionModel`
            parameter to choose which model to use.

            Required `empirical` model properties:
            * `freeFlowSpeedTable`

            Required `physical` model properties:
            * `driveEfficiency`
            * `recuperationEfficiency`

            When using the physical model, certain properties of the `vehicle` parameter are also
            required (see documentation for the `vehicle` parameter for more details):
            * `rollingResistanceCoefficient`
            * `airDragCoefficient`
            * `currentWeight`
            * `frontalArea` or combination of `width` and `height`
        fuel (Fuel | Unset): Fuel parameters to be used for calculating consumption and
            consumption-based isolines for fuel-based vehicles
            using empirical consumption model.

            Attributes:
            * `type`
            * `freeFlowSpeedTable`
            * `trafficSpeedTable`
            * `additionalConsumption`
            * `ascent`
        max_speed_on_segment (str | Unset): A comma separated list of segments with restrictions
            on maximum baseSpeed.

            Each entry has the following structure:
            `{segmentId}(#{direction})?;speed={maxBaseSpeed}`

            The individual parts are:
            * segmentId: The identifier of the referenced topology segment inside the catalog,
            example: `here:cm:segment:207551710`
            * direction (optional): Either '*' for bidirectional (default), '+' for positive
            direction, or '-' for negative direction
            * maxBaseSpeed: New value in m/s of baseSpeed on segment

            Example of a parameter value excluding two segments:
            `here:cm:segment:207551710#+;speed=10,here:cm:segment:76771992;speed=1`

            **Notes**:
            - It does not increase default baseSpeed on segment. If the value is greater than the
            default base speed, then such penalty will have no effect.
            - Minimum valid value for speed is 1
            - Using segments with a modified base speed does not trigger any notifications
            - Maximum number of penalized segments in one request cannot be greater than 1000.
              "penalized segments" refer to segments that have a restrictions on maximum baseSpeed
            with `maxSpeedOnSegment`
              or avoided with `avoid[segments]`
            - In case the same segment is penalized multiple times through values provided in the
            query string and/or the POST body,
              then the most restrictive value will be applied.
        taxi (Taxi | Unset): Taxi-specific parameters
        traffic (Traffic | Unset): Traffic specific parameters.
        billing_tag (str | Unset):  Example: ABCD+EFGH.
        x_request_id (str | Unset):
        body (CalculateIsolinesPostParameters | Unset): Parameters of the POST body for isoline
            calculation

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[AuthErrorResponseSchema | AuthErrorResponseSchema | NotAllowed | IsolineErrorResponse | IsolineResponse | ServerInternal]
    """

    kwargs = _get_kwargs(
        body=body,
        range_=range_,
        transport_mode=transport_mode,
        origin=origin,
        departure_time=departure_time,
        destination=destination,
        arrival_time=arrival_time,
        routing_mode=routing_mode,
        shape=shape,
        optimize_for=optimize_for,
        avoid=avoid,
        exclude=exclude,
        allow=allow,
        truck=truck,
        vehicle=vehicle,
        consumption_model=consumption_model,
        ev=ev,
        fuel=fuel,
        max_speed_on_segment=max_speed_on_segment,
        taxi=taxi,
        traffic=traffic,
        billing_tag=billing_tag,
        x_request_id=x_request_id,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient | Client,
    body: CalculateIsolinesPostParameters | Unset = UNSET,
    range_: Range,
    transport_mode: RouterMode,
    origin: str | Unset = UNSET,
    departure_time: datetime.datetime | Unset = UNSET,
    destination: str | Unset = UNSET,
    arrival_time: datetime.datetime | Unset = UNSET,
    routing_mode: RoutingMode | Unset = UNSET,
    shape: Shape | Unset = UNSET,
    optimize_for: CalculateIsolinesPostOptimizeFor
    | Unset = CalculateIsolinesPostOptimizeFor.BALANCED,
    avoid: Avoid | Unset = UNSET,
    exclude: Exclude | Unset = UNSET,
    allow: Allow | Unset = UNSET,
    truck: Truck | Unset = UNSET,
    vehicle: Vehicle | Unset = UNSET,
    consumption_model: str | Unset = UNSET,
    ev: BaseEVEmpiricalModel | BaseEVPhysicalModel | Unset = UNSET,
    fuel: Fuel | Unset = UNSET,
    max_speed_on_segment: str | Unset = UNSET,
    taxi: Taxi | Unset = UNSET,
    traffic: Traffic | Unset = UNSET,
    billing_tag: str | Unset = UNSET,
    x_request_id: str | Unset = UNSET,
) -> (
    AuthErrorResponseSchema
    | AuthErrorResponseSchema
    | NotAllowed
    | IsolineErrorResponse
    | IsolineResponse
    | ServerInternal
    | None
):
    """Calculate isolines via POST

     Retrieves a list of polylines that captures the approximate reachable area
    from the specified center point constrained by the provided parameters.

    At the moment, only select parameters are permitted in the POST payload. In particular, those
    parameters
    that due to request size may be limited in the query string. See the request body section below.
    These
    parameters can be provided either in the query string or in the POST body.
    All other parameters can only be provided in the query string at the moment.

    Args:
        range_ (Range): Ranges specified in terms of distance, travel time or energy consumption.
        transport_mode (RouterMode): Mode of transport to be used for route calculation.
        origin (str | Unset): A location defining an origin, destination or via point for a route
            or an isoline.

            Format: `Place[WaypointOptions]`

            * Place: `{lat},{lng}[PlaceOptions]`
            * PlaceOptions: `;option1=value1;option2=value2...`
            * WaypointOptions: `!option1=value1!option2=value2...`

            A waypoint consists of:
            * Exactly one place
            * Optional settings for the place
            * Optional settings for the waypoint itself

            Supported place options:
            * `course`: int, degrees clock-wise from north. Indicates the desired direction from the
            place. For example, `90` indicating `east`. Often combined with `radius` and/or
            `minCourseDistance`. This parameter takes preference over `matchSideOfStreet`.
            * `sideOfStreetHint`: `{lat},{lng}`. Indicates the side of the street that should be used.
            For example, if the location is to the left of the street, the router will prefer using
            that side in case the street has dividers. For example,
            `52.511496,13.304140;sideOfStreetHint=52.512149,13.304076` indicates that the `north` side
            of the street should be preferred. This options is required, if `matchSideOfStreet` is set
            to `always`. Option cannot be combined with `radius`, `radiusPenalty` or `snapRadius`.
            * `displayLocation`: `{lat},{lng}`. Indicates the physical location of the POI. It is
            different from the originalLocation and location which are generally expected to be on the
            navigable road network, i.e., on segments.
            * `uTurnPermission`: enum `[allow, avoid]`. Specifies the U-Turn Permission mode at the
            stop-over waypoint. If unspecified, the permission will be determined by the global
            setting, avoid[features]=uTurns. This feature is not supported for pass-through waypoints
            and U-Turns are generally avoided in that case.
              + `allow` : Allow making a U-Turn at this stop-over waypoint
              + `avoid` : Avoid making a U-Turn at this stop-over waypoint
            * `matchSideOfStreet`: enum `[always, onlyIfDivided]`. Specifies how the location set by
            `sideOfStreetHint` should be handled. Requires `sideOfStreetHint` to be specified as well.
            Note the exception above when combined with `course`.
              + `always` : Always prefer the given side of street.
              + `onlyIfDivided`: Only prefer using side of street set by `sideOfStreetHint` in case
            the street has dividers. This is the default behavior.
            * `nameHint`: string. Causes the router to look for the place with the most similar name.
            The typical examples include: `North` being used to differentiate between interstates `I66
            North` and `I66 South`, `Downtown Avenue` being used to correctly select a residental
            street.
            * `radius`: int, meters. Sets the radius within which all locations are considered equally
            eligible for selection as waypoints. Typical use cases include scenarios with imprecise
            coordinates or user input provided on low resolution map displays. The value is capped at
            200 meters. Setting a higher value doesn't return an error. Can't be used with
            `snapRadius`.
            * `snapRadius`: int, meters. Sets the radius within which waypoints are matched to the
            most "significant" place. Candidates for matching are sorted in the order of significance
            which is based on the visibility on a zoomed-out map. A highway is considered more
            significant than a national road, while a national road is more significant than a city
            road. Typical use case - selecting waypoints on a  zoomed-out view of a map on a drag-and-
            drop interface, where only significant roads are visible. A big enough radius allows to
            match waypoints to such roads. Can't be used with `radius` or `radiusPenalty` parameters.
            * `radiusPenalty`: int, percentage 0-10000. Penalty as percentage: Used in conjunction
            with the `radius` parameter. Router will match the waypoint within the specified radius
            and apply a penalty to candidates based on their air distance to the waypoint. This
            penalty is proportional to the given percentage, where 100 is just the cost of the air
            distance, and 200 is twice the cost of the air distance. The penalty must be chosen so
            that, when multiplied by the radius, the result is less than or equal to 7200. Regardless,
            only values up to and including 10000 will be accepted. This means that a maximum penalty
            of 3600% is allowed for a radius of 200m, 7200% for 100m and 10000% for 72m and less.
            Higher values will result in an error response. `radiusPenalty` cannot be combined with
            `snapRadius`. **Alpha**: This parameter is in development. It may not be stable and is
            subject to change.

            * `minCourseDistance`: int, meters. Instructs the routing service to try to find a route
            that avoids actions for the indicated distance. For example, if the origin is determined
            by a moving vehicle, the user might not have time to react to early actions. Values
            greater than 2000 meters will be capped at 2000 meters.
            * `segmentIdHint`: string. Causes the router to try and match to the specified segment.
            Waypoint coordinates need to be on the segment, otherwise waypoint will be matched
            ignoring the segment hint. This parameter can be used when the waypoint is too close to
            more than one segment to force matching to a specific one.
            * `onRoadThreshold`: int, meters. allows specifying a distance within which the waypoint
            could be considered as being on a highway/bridge/tunnel/sliproad. Within this threshold,
            the attributes of the segments do not impact the matching. Outside the threshold only
            segments which aren't one of highway/bridge/tunnel/sliproad can be matched.

            Supported waypoint options:
            * `stopDuration`: desired duration for the stop, in seconds.
               `stopDuration` must be less than 50000.
            * `passThrough`: boolean. Asks the router to avoid the following during route calculation:
              + Introducing a stop at the waypoint.
              + Splitting the route into sections.
              + Changing the direction of travel.

            Following scenarios are not supported for `passThrough` parameter:
              + Setting both `stopDuration` to a value greater than 0 and `passThrough=true`.
              + Setting `passThrough=true` for `origin` or `destination` of a route.
              The default value is `false`.
        departure_time (datetime.datetime | Unset): **RFC 3339**, section 5.6 as defined by either
            `date-time` or `date-only` 'T' `partial-time` (ie no time-offset).
        destination (str | Unset): A location defining an origin, destination or via point for a
            route or an isoline.

            Format: `Place[WaypointOptions]`

            * Place: `{lat},{lng}[PlaceOptions]`
            * PlaceOptions: `;option1=value1;option2=value2...`
            * WaypointOptions: `!option1=value1!option2=value2...`

            A waypoint consists of:
            * Exactly one place
            * Optional settings for the place
            * Optional settings for the waypoint itself

            Supported place options:
            * `course`: int, degrees clock-wise from north. Indicates the desired direction from the
            place. For example, `90` indicating `east`. Often combined with `radius` and/or
            `minCourseDistance`. This parameter takes preference over `matchSideOfStreet`.
            * `sideOfStreetHint`: `{lat},{lng}`. Indicates the side of the street that should be used.
            For example, if the location is to the left of the street, the router will prefer using
            that side in case the street has dividers. For example,
            `52.511496,13.304140;sideOfStreetHint=52.512149,13.304076` indicates that the `north` side
            of the street should be preferred. This options is required, if `matchSideOfStreet` is set
            to `always`. Option cannot be combined with `radius`, `radiusPenalty` or `snapRadius`.
            * `displayLocation`: `{lat},{lng}`. Indicates the physical location of the POI. It is
            different from the originalLocation and location which are generally expected to be on the
            navigable road network, i.e., on segments.
            * `uTurnPermission`: enum `[allow, avoid]`. Specifies the U-Turn Permission mode at the
            stop-over waypoint. If unspecified, the permission will be determined by the global
            setting, avoid[features]=uTurns. This feature is not supported for pass-through waypoints
            and U-Turns are generally avoided in that case.
              + `allow` : Allow making a U-Turn at this stop-over waypoint
              + `avoid` : Avoid making a U-Turn at this stop-over waypoint
            * `matchSideOfStreet`: enum `[always, onlyIfDivided]`. Specifies how the location set by
            `sideOfStreetHint` should be handled. Requires `sideOfStreetHint` to be specified as well.
            Note the exception above when combined with `course`.
              + `always` : Always prefer the given side of street.
              + `onlyIfDivided`: Only prefer using side of street set by `sideOfStreetHint` in case
            the street has dividers. This is the default behavior.
            * `nameHint`: string. Causes the router to look for the place with the most similar name.
            The typical examples include: `North` being used to differentiate between interstates `I66
            North` and `I66 South`, `Downtown Avenue` being used to correctly select a residental
            street.
            * `radius`: int, meters. Sets the radius within which all locations are considered equally
            eligible for selection as waypoints. Typical use cases include scenarios with imprecise
            coordinates or user input provided on low resolution map displays. The value is capped at
            200 meters. Setting a higher value doesn't return an error. Can't be used with
            `snapRadius`.
            * `snapRadius`: int, meters. Sets the radius within which waypoints are matched to the
            most "significant" place. Candidates for matching are sorted in the order of significance
            which is based on the visibility on a zoomed-out map. A highway is considered more
            significant than a national road, while a national road is more significant than a city
            road. Typical use case - selecting waypoints on a  zoomed-out view of a map on a drag-and-
            drop interface, where only significant roads are visible. A big enough radius allows to
            match waypoints to such roads. Can't be used with `radius` or `radiusPenalty` parameters.
            * `radiusPenalty`: int, percentage 0-10000. Penalty as percentage: Used in conjunction
            with the `radius` parameter. Router will match the waypoint within the specified radius
            and apply a penalty to candidates based on their air distance to the waypoint. This
            penalty is proportional to the given percentage, where 100 is just the cost of the air
            distance, and 200 is twice the cost of the air distance. The penalty must be chosen so
            that, when multiplied by the radius, the result is less than or equal to 7200. Regardless,
            only values up to and including 10000 will be accepted. This means that a maximum penalty
            of 3600% is allowed for a radius of 200m, 7200% for 100m and 10000% for 72m and less.
            Higher values will result in an error response. `radiusPenalty` cannot be combined with
            `snapRadius`. **Alpha**: This parameter is in development. It may not be stable and is
            subject to change.

            * `minCourseDistance`: int, meters. Instructs the routing service to try to find a route
            that avoids actions for the indicated distance. For example, if the origin is determined
            by a moving vehicle, the user might not have time to react to early actions. Values
            greater than 2000 meters will be capped at 2000 meters.
            * `segmentIdHint`: string. Causes the router to try and match to the specified segment.
            Waypoint coordinates need to be on the segment, otherwise waypoint will be matched
            ignoring the segment hint. This parameter can be used when the waypoint is too close to
            more than one segment to force matching to a specific one.
            * `onRoadThreshold`: int, meters. allows specifying a distance within which the waypoint
            could be considered as being on a highway/bridge/tunnel/sliproad. Within this threshold,
            the attributes of the segments do not impact the matching. Outside the threshold only
            segments which aren't one of highway/bridge/tunnel/sliproad can be matched.

            Supported waypoint options:
            * `stopDuration`: desired duration for the stop, in seconds.
               `stopDuration` must be less than 50000.
            * `passThrough`: boolean. Asks the router to avoid the following during route calculation:
              + Introducing a stop at the waypoint.
              + Splitting the route into sections.
              + Changing the direction of travel.

            Following scenarios are not supported for `passThrough` parameter:
              + Setting both `stopDuration` to a value greater than 0 and `passThrough=true`.
              + Setting `passThrough=true` for `origin` or `destination` of a route.
              The default value is `false`.
        arrival_time (datetime.datetime | Unset): **RFC 3339**, section 5.6 as defined by either
            `date-time` or `date-only` 'T' `partial-time` (ie no time-offset).
        routing_mode (RoutingMode | Unset): Specifies which optimization is applied during the
            calculation.

            * `fast`: Route calculation from start to destination optimized by travel time. In many
              cases, the route returned by `fast` mode may not be the route with the fastest
              possible travel time. For example, the routing service may favor a route that remains on
              a highway, even if a faster travel time can be achieved by taking a detour or shortcut
              through an inconvenient side road.
            * `short`: Route calculation from start to destination disregarding any speed information.
              In this mode, the distance of the route is minimized, while keeping the route sensible.
              This includes, for example, penalizing turns. Because of that, the resulting route will
              not necessarily be the one with minimal distance.

            Notes:
            * The following Transport modes only support `fast` routingMode
              - `bicycle`
              - `bus`
              - `pedestrian`
              - `privateBus`
              - `scooter`
              - `taxi`
        shape (Shape | Unset): A struct used to control the shape of the returned isolines.
        optimize_for (CalculateIsolinesPostOptimizeFor | Unset):  Default:
            CalculateIsolinesPostOptimizeFor.BALANCED.
        avoid (Avoid | Unset):
        exclude (Exclude | Unset): Options to exclude strictly during the route calculation.
        allow (Allow | Unset): Explicitly allow features that require users to opt in.
        truck (Truck | Unset): Vehicle-specific parameters
        vehicle (Vehicle | Unset): Vehicle-specific parameters
        consumption_model (str | Unset): Extensible enum: `empirical` `physical` `...`
            Specifies which of the EV consumption models is being used. See the `ev` parameter for
            details on the models.

            * `empirical`
            * `physical`

            **Alpha**: Physical consumption model is in development. It may not be stable and is
            subject to change.
        ev (BaseEVEmpiricalModel | BaseEVPhysicalModel | Unset): EV parameters to be used for
            calculating consumption and for calculating isolines for EV.

            There are two consumption models (`empirical`, `physical`). Set the `consumptionModel`
            parameter to choose which model to use.

            Required `empirical` model properties:
            * `freeFlowSpeedTable`

            Required `physical` model properties:
            * `driveEfficiency`
            * `recuperationEfficiency`

            When using the physical model, certain properties of the `vehicle` parameter are also
            required (see documentation for the `vehicle` parameter for more details):
            * `rollingResistanceCoefficient`
            * `airDragCoefficient`
            * `currentWeight`
            * `frontalArea` or combination of `width` and `height`
        fuel (Fuel | Unset): Fuel parameters to be used for calculating consumption and
            consumption-based isolines for fuel-based vehicles
            using empirical consumption model.

            Attributes:
            * `type`
            * `freeFlowSpeedTable`
            * `trafficSpeedTable`
            * `additionalConsumption`
            * `ascent`
        max_speed_on_segment (str | Unset): A comma separated list of segments with restrictions
            on maximum baseSpeed.

            Each entry has the following structure:
            `{segmentId}(#{direction})?;speed={maxBaseSpeed}`

            The individual parts are:
            * segmentId: The identifier of the referenced topology segment inside the catalog,
            example: `here:cm:segment:207551710`
            * direction (optional): Either '*' for bidirectional (default), '+' for positive
            direction, or '-' for negative direction
            * maxBaseSpeed: New value in m/s of baseSpeed on segment

            Example of a parameter value excluding two segments:
            `here:cm:segment:207551710#+;speed=10,here:cm:segment:76771992;speed=1`

            **Notes**:
            - It does not increase default baseSpeed on segment. If the value is greater than the
            default base speed, then such penalty will have no effect.
            - Minimum valid value for speed is 1
            - Using segments with a modified base speed does not trigger any notifications
            - Maximum number of penalized segments in one request cannot be greater than 1000.
              "penalized segments" refer to segments that have a restrictions on maximum baseSpeed
            with `maxSpeedOnSegment`
              or avoided with `avoid[segments]`
            - In case the same segment is penalized multiple times through values provided in the
            query string and/or the POST body,
              then the most restrictive value will be applied.
        taxi (Taxi | Unset): Taxi-specific parameters
        traffic (Traffic | Unset): Traffic specific parameters.
        billing_tag (str | Unset):  Example: ABCD+EFGH.
        x_request_id (str | Unset):
        body (CalculateIsolinesPostParameters | Unset): Parameters of the POST body for isoline
            calculation

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        AuthErrorResponseSchema | AuthErrorResponseSchema | NotAllowed | IsolineErrorResponse | IsolineResponse | ServerInternal
    """

    return (
        await asyncio_detailed(
            client=client,
            body=body,
            range_=range_,
            transport_mode=transport_mode,
            origin=origin,
            departure_time=departure_time,
            destination=destination,
            arrival_time=arrival_time,
            routing_mode=routing_mode,
            shape=shape,
            optimize_for=optimize_for,
            avoid=avoid,
            exclude=exclude,
            allow=allow,
            truck=truck,
            vehicle=vehicle,
            consumption_model=consumption_model,
            ev=ev,
            fuel=fuel,
            max_speed_on_segment=max_speed_on_segment,
            taxi=taxi,
            traffic=traffic,
            billing_tag=billing_tag,
            x_request_id=x_request_id,
        )
    ).parsed



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.max_speed_type_1 import MaxSpeedType1
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.dynamic_speed_info import DynamicSpeedInfo
    from ..models.localized_route_number import LocalizedRouteNumber
    from ..models.localized_string import LocalizedString


T = TypeVar("T", bound="VehicleSpan")


@_attrs_define
class VehicleSpan:
    r"""Span attached to a `Section` that describes the vehicle content.

    Attributes:
        offset (int | Unset): Offset of a coordinate in the section's polyline.
        length (int | Unset): Distance in meters. Example: 189.
        duration (int | Unset): Duration in seconds. Example: 198.
        street_attributes (list[str] | Unset): `StreetAttributes` is applied to a span of a route section and describes
            attribute flags of a street. The following flags can be assigned:
            * `rightDrivingSide`: Vehicles must drive on the right-hand side of the road in this part of the route.
            * `dirtRoad`: This part of the route has an unpaved surface, such as a gravel or dirt road.
            * `tunnel`: This part of the route passes through a tunnel.
            * `bridge`: This part of the route crosses a bridge.
            * `ramp`: This part of the route is a ramp, typically connecting to or from highways.
            * `motorway`: This part of the route is a controlled access road, typically a highway with motorway signage.
            * `roundabout`: This part of the route includes a roundabout.
            * `underConstruction`: This part of the route is currently under construction.
            * `dividedRoad`: This part of the route uses a road with a physical or legal divider in the middle, separating
            opposing traffic flows.
            * `privateRoad`: This part of the route uses a privately owned road.
            * `controlledAccessHighway`: This part of the route is either a controlled access road, limited access road, or
            both.
            * `builtUpArea`: This part of the route most likely passes through a built-up area.

            It is possible that new street-attributes may be supported in the future. When encountering unknown street-
            attributes, e.g., in the software parser, it is recommended to ignore them.
        walk_attributes (list[str] | Unset): The following attribute flags are used to describe accessibility and walk-
            related features along a route:

            * `stairs`: This part of the route includes a staircase that needs to be climbed or descended.
            * `park`: This part of the route is located within a park or park-like area.
            * `indoor`: This part of the route is inside a venue or building.
            * `open`: This part of the route is open and accessible for walking.
            * `noThrough`: This part of the route can only be traversed if the origin, destination, or any via waypoint is
            located there. In other words, it is not a through-route.
            * `tollRoad`: Access to this part of the route is restricted and requires the payment of a fee or toll.

            It is possible that new street-attributes may be supported in the future. When encountering unknown street-
            attributes, e.g., in the software parser, it is recommended to ignore them.
        car_attributes (list[str] | Unset): Car-specific `AccessAttributes`.

            `AccessAttributes` is applied to a span of a route section and describe access flags specific to cars. The
            following flags can be assigned:
            * `open`: This part of the route is open and accessible to cars.
            * `noThrough`: This part of the route can only be traversed by cars if the origin, destination, or any via
            waypoint is located there. It is not a through-route for cars.
            * `tollRoad`: This part of the route is a toll road, which means access is restricted for cars and requires the
            payment of a fee or toll.

            It is possible that new street-attributes may be supported in the future. When encountering unknown street-
            attributes, e.g., in the software parser, it is recommended to ignore them.
        truck_attributes (list[str] | Unset): Truck-specific `AccessAttributes`.

            `AccessAttributes` is applied to a span of a route section and describe access flags specific to trucks. These
            flags indicate the accessibility and restrictions for trucks along the route. The following flags can be
            assigned:
            * `open`: This part of the route is open and accessible to trucks.
            * `noThrough`: That this part of the route can only be traversed by trucks if the origin, destination, or any
            via waypoint is located there. It is not a through-route for trucks.
            * `tollRoad`: This part of the route is a toll road, which means access is restricted for trucks and requires
            the payment of a fee or toll.

            It is possible that new street-attributes may be supported in the future. When encountering unknown street-
            attributes, e.g., in the software parser, it is recommended to ignore them.
        scooter_attributes (list[str] | Unset): Scooter-specific `AccessAttributes`.

            `AccessAttributes` is applied to a span of a route section and describe access flags specific to scooters. These
            flags indicate the accessibility of streets for scooters along the route. The following flag can be assigned:
            * `open`: Indicates that this part of the route is open and accessible to scooters.

            It is possible that new street-attributes may be supported in the future. When encountering unknown street-
            attributes, e.g., in the software parser, it is recommended to ignore them.
        toll_systems (list[int] | Unset): The toll systems applicable to specific spans are described using an array of
            reference indexes. These indexes correspond to the `tollSystems` array in the enclosing section. Please note
            that toll information is not static and can vary based on factors such as transport-mode, time-of-day, etc.
            Therefore, the tolls applicable to a span may change in similar requests.
        names (list[LocalizedString] | Unset): Designated name for the span, e.g., a street name or a transport name.
        route_numbers (list[LocalizedRouteNumber] | Unset): Designated route name or number of the span, e.g., 'M25')
        country_code (str | Unset): ISO-3166-1 alpha-3 code Example: FRA.
        state_code (str | Unset): The second part of an ISO-3166-2 code (e.g., `TX` from `USA-TX`) consists of up to
            three alphanumeric characters.
            It is used to identify the principal subdivisions (e.g., provinces or states) of a country in conjunction with a
            CountryCode

            Note: State codes may not be available in some countries.
        functional_class (int | Unset): Functional class defines a hierarchical network used to determine a logical and
            efficient route. The following classifications are used:

            * `1`: Roads that allow for high volume, maximum speed traffic movement between and through major metropolitan
            areas.
            * `2`: Roads that are used to channel traffic to functional class 1 roads for travel between and through cities
            in the shortest amount of time.
            * `3`: Roads that intersect functional class 2 roads and provide a high volume of traffic movement at a lower
            level of mobility than functional class 2 roads.
            * `4`: Roads that provide for a high volume of traffic movement at moderate speeds between neighborhoods.
            * `5`: Roads with volume and traffic movement below the level of any other functional class.
        speed_limit (float | Unset): Speed in meters per second
        max_speed (float | MaxSpeedType1 | Unset): Speed in meters per second, or "unlimited" indicating that the speed
            is unlimited, e.g., on a German autobahn
        dynamic_speed_info (DynamicSpeedInfo | Unset): Describes dynamic speed information, such as traffic speed,
            estimated speed without traffic, and turn time.
        segment_id (str | Unset):
            **NOTE:** The attribute `segmentId` is deprecated and should be replaced with `segmentRef`.

            The directed topology segment id, including prefix, e.g., '+here:cm:segment:'. The id consists of two parts
            * The direction, represented by either '+' or '-'.
            * The topology segment id (a unique identifier within the HERE platform catalogs).

            The direction specifies whether the route is using the segment in its canonical direction ('+' also known as
            traveling along the geometry's direction), or against it ('-' also known as traveling against the geometry's
            direction).
        segment_ref (str | Unset): A reference to the HMC topology segment used in this span.

            The standard representation of a segment reference has the following structure:
            {catalogHrn}:{catalogVersion}:({layerId})?:{tileId}:{segmentId}(#{direction}({startOffset}..{endOffset})?)?

            The individual parts are:
            * catalogHrn: The HERE Resource Name that identifies the source catalog of the segment, example:
            hrn:here:data::olp-here:rib-2
            * catalogVersion: The catalog version
            * layerId (optional): The layer inside the catalog where the segment can be found, example: topology-geometry
            * tileId: The HERE tile key of the partition/tile where the segment is located in the given version of the
            catalog. This can be on a lower level than the actual segment is stored at (for example, the provided tile ID
            can be on level 14, despite topology-geometry partitions being tiled at level 12). The level of a HERE tile key
            is indicated by the position of the highest set bit in binary representation. Since the HERE tile key represents
            a morton code of the x and y portion of the Tile ID, the level 12 tile ID can be retrieved from the level 14
            tile ID by removing the 4 least significant bits (or 2 bits per level) or 1 hexadecimal digit. For example, the
            level 14 tile 377894441 is included in the level 12 tile 23618402 (377894441<sub>10</sub> =
            16863629<sub>16</sub> &rightarrow; 1686362<sub>16</sub> = 23618402<sub>10</sub>)
            * segmentId: The identifier of the referenced topology segment inside the catalog, example:
            here:cm:segment:84905195
            * direction (optional): Either '*' for undirected or bidirectional, '+' for positive direction, '-' for negative
            direction, or '?' for unknown direction (not used by the routing service)
            * startOffset/endOffset (optional): The start- and end offset are non-negative numbers between 0 and 1,
            representing the start and end of the referenced range using a proportion of the length of the segment. 0
            represents the start and 1 the end of the segment, relative to the indicated direction (or positive direction in
            case of undirected segments). Example: 0.7..1

            Example of a segment reference in standard representation:
            hrn:here:data::olp-here:rib-2:1363::377894441:here:cm:segment:84905195#+0.7..1

            The segment references can also be provided in a compact representation, to reduce the response size. In the
            compact representation, some parts are replaced by placeholders, which can be resolved using the refReplacements
            dictionary in the parent section.
            The placeholder format is ```\$\d+``` and needs to be surrounded by colons or string start/end. It can be
            captured with the following regular expression: ```(^|:)\$\d+(:|$)/``` .

            Example of the segment reference previously mentioned in compact representation:
            $0:377894441:$1:84905195#+0.7..1
            With the corresponding refReplacements:
            "refReplacements": {
              "0": "hrn:here:data::olp-here:rib-2:1363:",
              "1": "here:cm:segment"
            }
        consumption (float | Unset): Energy or fuel consumption.
            For EV energy consumption is in kilowatt hours (kWh). For fuel-based vehicles fuel consumption is in Liters (L)
            for diesel, petrol and LPG vehicles, and Kilograms (kg) for CNG vehicles.
        base_duration (int | Unset): Duration in seconds. Example: 198.
        typical_duration (int | Unset): Duration in seconds. Example: 198.
        incidents (list[int] | Unset): A list of indexes into the incident array of the parent section.
            References all incidents that apply to the span.
            This requires `incidents` to be specified as part of the `return` parameter.
        intersection_incidents (list[int] | Unset): A list of indexes in the incident array of the parent section.
            References all incidents that block movement through the intersection at the end of this span.
            This requires `incidents` to be specified in `return` and `spans` parameters.
        routing_zones (list[int] | Unset): A list of indexes into the routing zone array of the parent section.
            References all applicable routing zones to the span.
        truck_road_types (list[int] | Unset): A list of indexes into the truck road types array of the parent section.
            References all applicable truck road types to the span.
        gate (str | Unset): Extensible enum: `keyAccess` `permissionRequired` `emergency` `...`
            Type of gate crossing.
        railway_crossing (str | Unset): Extensible enum: `protected` `unprotected` `unknown` `...`
            Type of railway crossing.
        notices (list[int] | Unset): A list of indexes into the notices array of the parent section.
            References all notices that apply to the span.

            Depending on the notice type the notice might apply to the whole span, or only at the end of the span. Notices
            that only apply at the end of the span are:

            | Code      | Description  |
            | --------- | ------- |
            | violatedTurnRestriction | A restricted turn is performed at the end of the span |
            | violatedAvoidDifficultTurns | A difficult turn is performed at the end of the span |
            | violatedAvoidUTurns | A U-turn is performed at the end of the span |
            | violatedEmergencyGate | An emergency gate is crossed at the end of the span |
        no_through_restrictions (list[int] | Unset): A list of indexes into the `noThroughRestrictions` array of the
            parent section.
            References all `noThroughRestrictions` that apply to the span.
    """

    offset: int | Unset = UNSET
    length: int | Unset = UNSET
    duration: int | Unset = UNSET
    street_attributes: list[str] | Unset = UNSET
    walk_attributes: list[str] | Unset = UNSET
    car_attributes: list[str] | Unset = UNSET
    truck_attributes: list[str] | Unset = UNSET
    scooter_attributes: list[str] | Unset = UNSET
    toll_systems: list[int] | Unset = UNSET
    names: list[LocalizedString] | Unset = UNSET
    route_numbers: list[LocalizedRouteNumber] | Unset = UNSET
    country_code: str | Unset = UNSET
    state_code: str | Unset = UNSET
    functional_class: int | Unset = UNSET
    speed_limit: float | Unset = UNSET
    max_speed: float | MaxSpeedType1 | Unset = UNSET
    dynamic_speed_info: DynamicSpeedInfo | Unset = UNSET
    segment_id: str | Unset = UNSET
    segment_ref: str | Unset = UNSET
    consumption: float | Unset = UNSET
    base_duration: int | Unset = UNSET
    typical_duration: int | Unset = UNSET
    incidents: list[int] | Unset = UNSET
    intersection_incidents: list[int] | Unset = UNSET
    routing_zones: list[int] | Unset = UNSET
    truck_road_types: list[int] | Unset = UNSET
    gate: str | Unset = UNSET
    railway_crossing: str | Unset = UNSET
    notices: list[int] | Unset = UNSET
    no_through_restrictions: list[int] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        offset = self.offset

        length = self.length

        duration = self.duration

        street_attributes: list[str] | Unset = UNSET
        if not isinstance(self.street_attributes, Unset):
            street_attributes = self.street_attributes

        walk_attributes: list[str] | Unset = UNSET
        if not isinstance(self.walk_attributes, Unset):
            walk_attributes = self.walk_attributes

        car_attributes: list[str] | Unset = UNSET
        if not isinstance(self.car_attributes, Unset):
            car_attributes = self.car_attributes

        truck_attributes: list[str] | Unset = UNSET
        if not isinstance(self.truck_attributes, Unset):
            truck_attributes = self.truck_attributes

        scooter_attributes: list[str] | Unset = UNSET
        if not isinstance(self.scooter_attributes, Unset):
            scooter_attributes = self.scooter_attributes

        toll_systems: list[int] | Unset = UNSET
        if not isinstance(self.toll_systems, Unset):
            toll_systems = self.toll_systems

        names: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.names, Unset):
            names = []
            for names_item_data in self.names:
                names_item = names_item_data.to_dict()
                names.append(names_item)

        route_numbers: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.route_numbers, Unset):
            route_numbers = []
            for route_numbers_item_data in self.route_numbers:
                route_numbers_item = route_numbers_item_data.to_dict()
                route_numbers.append(route_numbers_item)

        country_code = self.country_code

        state_code = self.state_code

        functional_class = self.functional_class

        speed_limit = self.speed_limit

        max_speed: float | str | Unset
        if isinstance(self.max_speed, Unset):
            max_speed = UNSET
        elif isinstance(self.max_speed, MaxSpeedType1):
            max_speed = self.max_speed.value
        else:
            max_speed = self.max_speed

        dynamic_speed_info: dict[str, Any] | Unset = UNSET
        if not isinstance(self.dynamic_speed_info, Unset):
            dynamic_speed_info = self.dynamic_speed_info.to_dict()

        segment_id = self.segment_id

        segment_ref = self.segment_ref

        consumption = self.consumption

        base_duration = self.base_duration

        typical_duration = self.typical_duration

        incidents: list[int] | Unset = UNSET
        if not isinstance(self.incidents, Unset):
            incidents = self.incidents

        intersection_incidents: list[int] | Unset = UNSET
        if not isinstance(self.intersection_incidents, Unset):
            intersection_incidents = self.intersection_incidents

        routing_zones: list[int] | Unset = UNSET
        if not isinstance(self.routing_zones, Unset):
            routing_zones = self.routing_zones

        truck_road_types: list[int] | Unset = UNSET
        if not isinstance(self.truck_road_types, Unset):
            truck_road_types = self.truck_road_types

        gate = self.gate

        railway_crossing = self.railway_crossing

        notices: list[int] | Unset = UNSET
        if not isinstance(self.notices, Unset):
            notices = self.notices

        no_through_restrictions: list[int] | Unset = UNSET
        if not isinstance(self.no_through_restrictions, Unset):
            no_through_restrictions = self.no_through_restrictions

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if offset is not UNSET:
            field_dict["offset"] = offset
        if length is not UNSET:
            field_dict["length"] = length
        if duration is not UNSET:
            field_dict["duration"] = duration
        if street_attributes is not UNSET:
            field_dict["streetAttributes"] = street_attributes
        if walk_attributes is not UNSET:
            field_dict["walkAttributes"] = walk_attributes
        if car_attributes is not UNSET:
            field_dict["carAttributes"] = car_attributes
        if truck_attributes is not UNSET:
            field_dict["truckAttributes"] = truck_attributes
        if scooter_attributes is not UNSET:
            field_dict["scooterAttributes"] = scooter_attributes
        if toll_systems is not UNSET:
            field_dict["tollSystems"] = toll_systems
        if names is not UNSET:
            field_dict["names"] = names
        if route_numbers is not UNSET:
            field_dict["routeNumbers"] = route_numbers
        if country_code is not UNSET:
            field_dict["countryCode"] = country_code
        if state_code is not UNSET:
            field_dict["stateCode"] = state_code
        if functional_class is not UNSET:
            field_dict["functionalClass"] = functional_class
        if speed_limit is not UNSET:
            field_dict["speedLimit"] = speed_limit
        if max_speed is not UNSET:
            field_dict["maxSpeed"] = max_speed
        if dynamic_speed_info is not UNSET:
            field_dict["dynamicSpeedInfo"] = dynamic_speed_info
        if segment_id is not UNSET:
            field_dict["segmentId"] = segment_id
        if segment_ref is not UNSET:
            field_dict["segmentRef"] = segment_ref
        if consumption is not UNSET:
            field_dict["consumption"] = consumption
        if base_duration is not UNSET:
            field_dict["baseDuration"] = base_duration
        if typical_duration is not UNSET:
            field_dict["typicalDuration"] = typical_duration
        if incidents is not UNSET:
            field_dict["incidents"] = incidents
        if intersection_incidents is not UNSET:
            field_dict["intersectionIncidents"] = intersection_incidents
        if routing_zones is not UNSET:
            field_dict["routingZones"] = routing_zones
        if truck_road_types is not UNSET:
            field_dict["truckRoadTypes"] = truck_road_types
        if gate is not UNSET:
            field_dict["gate"] = gate
        if railway_crossing is not UNSET:
            field_dict["railwayCrossing"] = railway_crossing
        if notices is not UNSET:
            field_dict["notices"] = notices
        if no_through_restrictions is not UNSET:
            field_dict["noThroughRestrictions"] = no_through_restrictions

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.dynamic_speed_info import DynamicSpeedInfo
        from ..models.localized_route_number import LocalizedRouteNumber
        from ..models.localized_string import LocalizedString

        d = dict(src_dict)
        offset = d.pop("offset", UNSET)

        length = d.pop("length", UNSET)

        duration = d.pop("duration", UNSET)

        street_attributes = cast(list[str], d.pop("streetAttributes", UNSET))

        walk_attributes = cast(list[str], d.pop("walkAttributes", UNSET))

        car_attributes = cast(list[str], d.pop("carAttributes", UNSET))

        truck_attributes = cast(list[str], d.pop("truckAttributes", UNSET))

        scooter_attributes = cast(list[str], d.pop("scooterAttributes", UNSET))

        toll_systems = cast(list[int], d.pop("tollSystems", UNSET))

        _names = d.pop("names", UNSET)
        names: list[LocalizedString] | Unset = UNSET
        if _names is not UNSET:
            names = []
            for names_item_data in _names:
                names_item = LocalizedString.from_dict(names_item_data)

                names.append(names_item)

        _route_numbers = d.pop("routeNumbers", UNSET)
        route_numbers: list[LocalizedRouteNumber] | Unset = UNSET
        if _route_numbers is not UNSET:
            route_numbers = []
            for route_numbers_item_data in _route_numbers:
                route_numbers_item = LocalizedRouteNumber.from_dict(
                    route_numbers_item_data
                )

                route_numbers.append(route_numbers_item)

        country_code = d.pop("countryCode", UNSET)

        state_code = d.pop("stateCode", UNSET)

        functional_class = d.pop("functionalClass", UNSET)

        speed_limit = d.pop("speedLimit", UNSET)

        def _parse_max_speed(data: object) -> float | MaxSpeedType1 | Unset:
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                componentsschemas_max_speed_type_1 = MaxSpeedType1(data)

                return componentsschemas_max_speed_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(float | MaxSpeedType1 | Unset, data)

        max_speed = _parse_max_speed(d.pop("maxSpeed", UNSET))

        _dynamic_speed_info = d.pop("dynamicSpeedInfo", UNSET)
        dynamic_speed_info: DynamicSpeedInfo | Unset
        if isinstance(_dynamic_speed_info, Unset):
            dynamic_speed_info = UNSET
        else:
            dynamic_speed_info = DynamicSpeedInfo.from_dict(_dynamic_speed_info)

        segment_id = d.pop("segmentId", UNSET)

        segment_ref = d.pop("segmentRef", UNSET)

        consumption = d.pop("consumption", UNSET)

        base_duration = d.pop("baseDuration", UNSET)

        typical_duration = d.pop("typicalDuration", UNSET)

        incidents = cast(list[int], d.pop("incidents", UNSET))

        intersection_incidents = cast(list[int], d.pop("intersectionIncidents", UNSET))

        routing_zones = cast(list[int], d.pop("routingZones", UNSET))

        truck_road_types = cast(list[int], d.pop("truckRoadTypes", UNSET))

        gate = d.pop("gate", UNSET)

        railway_crossing = d.pop("railwayCrossing", UNSET)

        notices = cast(list[int], d.pop("notices", UNSET))

        no_through_restrictions = cast(list[int], d.pop("noThroughRestrictions", UNSET))

        vehicle_span = cls(
            offset=offset,
            length=length,
            duration=duration,
            street_attributes=street_attributes,
            walk_attributes=walk_attributes,
            car_attributes=car_attributes,
            truck_attributes=truck_attributes,
            scooter_attributes=scooter_attributes,
            toll_systems=toll_systems,
            names=names,
            route_numbers=route_numbers,
            country_code=country_code,
            state_code=state_code,
            functional_class=functional_class,
            speed_limit=speed_limit,
            max_speed=max_speed,
            dynamic_speed_info=dynamic_speed_info,
            segment_id=segment_id,
            segment_ref=segment_ref,
            consumption=consumption,
            base_duration=base_duration,
            typical_duration=typical_duration,
            incidents=incidents,
            intersection_incidents=intersection_incidents,
            routing_zones=routing_zones,
            truck_road_types=truck_road_types,
            gate=gate,
            railway_crossing=railway_crossing,
            notices=notices,
            no_through_restrictions=no_through_restrictions,
        )

        vehicle_span.additional_properties = d
        return vehicle_span

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.match_trace_point import MatchTracePoint
    from ..models.match_trace_via import MatchTraceVia


T = TypeVar("T", bound="MatchTrace")


@_attrs_define
class MatchTrace:
    """Trace file with points and path match parameters

    Example:
        {'trace': [{'lat': 52.0, 'lng': 13.1}, {'lat': 52.1, 'lng': 13.2}, {'lat': 52.2, 'lng': 13.3}], 'via':
            [{'index': 1}]}

    Attributes:
        trace (list[MatchTracePoint]):
        via (list[MatchTraceVia] | Unset):
    """

    trace: list[MatchTracePoint]
    via: list[MatchTraceVia] | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        trace = []
        for trace_item_data in self.trace:
            trace_item = trace_item_data.to_dict()
            trace.append(trace_item)

        via: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.via, Unset):
            via = []
            for via_item_data in self.via:
                via_item = via_item_data.to_dict()
                via.append(via_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({
            "trace": trace,
        })
        if via is not UNSET:
            field_dict["via"] = via

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.match_trace_point import MatchTracePoint
        from ..models.match_trace_via import MatchTraceVia

        d = dict(src_dict)
        trace = []
        _trace = d.pop("trace")
        for trace_item_data in _trace:
            trace_item = MatchTracePoint.from_dict(trace_item_data)

            trace.append(trace_item)

        _via = d.pop("via", UNSET)
        via: list[MatchTraceVia] | Unset = UNSET
        if _via is not UNSET:
            via = []
            for via_item_data in _via:
                via_item = MatchTraceVia.from_dict(via_item_data)

                via.append(via_item)

        match_trace = cls(
            trace=trace,
            via=via,
        )

        match_trace.additional_properties = d
        return match_trace

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.bounding_box_area_with_exceptions import BoundingBoxAreaWithExceptions
    from ..models.corridor_area_with_exceptions import CorridorAreaWithExceptions
    from ..models.encoded_corridor_area_with_exceptions import (
        EncodedCorridorAreaWithExceptions,
    )
    from ..models.encoded_polygon_area_with_exceptions import (
        EncodedPolygonAreaWithExceptions,
    )
    from ..models.polygon_area_with_exceptions import PolygonAreaWithExceptions


T = TypeVar("T", bound="AvoidPost")


@_attrs_define
class AvoidPost:
    """Avoid routes that violate certain features of road network or that go through
    user-specified geographical bounding boxes.

    For the general description of the functionality please refer to the `avoid` parameter of the
    query string.

    Passing parameters in the POST body is suggested when the length of the parameters exceeds the
    limitation of the GET request.

        Attributes:
            segments (list[str] | Unset): An array of segment identifiers that routes will avoid going through.

                Each entry has the following structure:
                `{segmentId}(#{direction})?`

                The individual parts are:
                * segmentId: The identifier of the referenced topology segment inside the catalog, example:
                `here:cm:segment:207551710`
                * direction (optional): Either '*' for bidirectional (default), '+' for positive direction, or '-' for negative
                direction

                Example of a parameter value excluding two segments:
                `["here:cm:segment:207551710#+", "here:cm:segment:76771992#*"]`

                **Note**: Maximum number of penalized segments in one request should not be greater than 1000.
                          "Penalized segments" refers to segments that have a restriction on maximum baseSpeed with
                `maxSpeedOnSegment`
                          or avoided with `avoid[segments]`
            areas (list[BoundingBoxAreaWithExceptions | CorridorAreaWithExceptions | EncodedCorridorAreaWithExceptions |
                EncodedPolygonAreaWithExceptions | PolygonAreaWithExceptions] | Unset): List of user-defined areas that routes
                should avoid/exclude going through.

                Notes:
                * Maximum count of avoided and excluded polygons and corridors is 20.
                * Maximum total count of avoided and excluded bounding boxes, polygons, corridors, including exceptions, is 250.
                 Example: {'areas': [{'type': 'polygon', 'outer': [{'lat': 52.514414, 'lng': 13.384685}, {'lat': 52.514414,
                'lng': 13.393568}, {'lat': 52.512403, 'lng': 13.393568}, {'lat': 52.512403, 'lng': 13.384685}]}, {'type':
                'encodedPolygon', 'outer': 'BG8mnlkD6-9wZAmrR19DAAlrR'}, {'type': 'boundingBox', 'west': 1.1, 'south': 2.2,
                'east': 3.3, 'north': 4.4, 'exceptions': [{'type': 'polygon', 'outer': [{'lat': 52.514414, 'lng': 13.384685},
                {'lat': 52.514414, 'lng': 13.393568}, {'lat': 52.512403, 'lng': 13.393568}, {'lat': 52.512403, 'lng':
                13.384685}]}, {'type': 'encodedPolygon', 'outer': 'BG8mnlkD6-9wZAmrR19DAAlrR'}]}]}.
    """

    segments: list[str] | Unset = UNSET
    areas: (
        list[
            BoundingBoxAreaWithExceptions
            | CorridorAreaWithExceptions
            | EncodedCorridorAreaWithExceptions
            | EncodedPolygonAreaWithExceptions
            | PolygonAreaWithExceptions
        ]
        | Unset
    ) = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.bounding_box_area_with_exceptions import (
            BoundingBoxAreaWithExceptions,
        )
        from ..models.corridor_area_with_exceptions import CorridorAreaWithExceptions
        from ..models.encoded_polygon_area_with_exceptions import (
            EncodedPolygonAreaWithExceptions,
        )
        from ..models.polygon_area_with_exceptions import PolygonAreaWithExceptions

        segments: list[str] | Unset = UNSET
        if not isinstance(self.segments, Unset):
            segments = self.segments

        areas: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.areas, Unset):
            areas = []
            for componentsschemas_areas_post_item_data in self.areas:
                componentsschemas_areas_post_item: dict[str, Any]
                if isinstance(
                    componentsschemas_areas_post_item_data,
                    BoundingBoxAreaWithExceptions,
                ):
                    componentsschemas_areas_post_item = (
                        componentsschemas_areas_post_item_data.to_dict()
                    )
                elif isinstance(
                    componentsschemas_areas_post_item_data, PolygonAreaWithExceptions
                ):
                    componentsschemas_areas_post_item = (
                        componentsschemas_areas_post_item_data.to_dict()
                    )
                elif isinstance(
                    componentsschemas_areas_post_item_data,
                    EncodedPolygonAreaWithExceptions,
                ):
                    componentsschemas_areas_post_item = (
                        componentsschemas_areas_post_item_data.to_dict()
                    )
                elif isinstance(
                    componentsschemas_areas_post_item_data, CorridorAreaWithExceptions
                ):
                    componentsschemas_areas_post_item = (
                        componentsschemas_areas_post_item_data.to_dict()
                    )
                else:
                    componentsschemas_areas_post_item = (
                        componentsschemas_areas_post_item_data.to_dict()
                    )

                areas.append(componentsschemas_areas_post_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if segments is not UNSET:
            field_dict["segments"] = segments
        if areas is not UNSET:
            field_dict["areas"] = areas

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.bounding_box_area_with_exceptions import (
            BoundingBoxAreaWithExceptions,
        )
        from ..models.corridor_area_with_exceptions import CorridorAreaWithExceptions
        from ..models.encoded_corridor_area_with_exceptions import (
            EncodedCorridorAreaWithExceptions,
        )
        from ..models.encoded_polygon_area_with_exceptions import (
            EncodedPolygonAreaWithExceptions,
        )
        from ..models.polygon_area_with_exceptions import PolygonAreaWithExceptions

        d = dict(src_dict)
        segments = cast(list[str], d.pop("segments", UNSET))

        _areas = d.pop("areas", UNSET)
        areas: (
            list[
                BoundingBoxAreaWithExceptions
                | CorridorAreaWithExceptions
                | EncodedCorridorAreaWithExceptions
                | EncodedPolygonAreaWithExceptions
                | PolygonAreaWithExceptions
            ]
            | Unset
        ) = UNSET
        if _areas is not UNSET:
            areas = []
            for componentsschemas_areas_post_item_data in _areas:

                def _parse_componentsschemas_areas_post_item(
                    data: object,
                ) -> (
                    BoundingBoxAreaWithExceptions
                    | CorridorAreaWithExceptions
                    | EncodedCorridorAreaWithExceptions
                    | EncodedPolygonAreaWithExceptions
                    | PolygonAreaWithExceptions
                ):
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_area_post_type_0 = (
                            BoundingBoxAreaWithExceptions.from_dict(data)
                        )

                        return componentsschemas_area_post_type_0
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_area_post_type_1 = (
                            PolygonAreaWithExceptions.from_dict(data)
                        )

                        return componentsschemas_area_post_type_1
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_area_post_type_2 = (
                            EncodedPolygonAreaWithExceptions.from_dict(data)
                        )

                        return componentsschemas_area_post_type_2
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        componentsschemas_area_post_type_3 = (
                            CorridorAreaWithExceptions.from_dict(data)
                        )

                        return componentsschemas_area_post_type_3
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    if not isinstance(data, dict):
                        raise TypeError()
                    componentsschemas_area_post_type_4 = (
                        EncodedCorridorAreaWithExceptions.from_dict(data)
                    )

                    return componentsschemas_area_post_type_4

                componentsschemas_areas_post_item = (
                    _parse_componentsschemas_areas_post_item(
                        componentsschemas_areas_post_item_data
                    )
                )

                areas.append(componentsschemas_areas_post_item)

        avoid_post = cls(
            segments=segments,
            areas=areas,
        )

        avoid_post.additional_properties = d
        return avoid_post

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
